'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _Coordinates = require('../Geographic/Coordinates');

var _Coordinates2 = _interopRequireDefault(_Coordinates);

var _Projection = require('../Geographic/Projection');

var _Projection2 = _interopRequireDefault(_Projection);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Extent is a SIG-area (so 2D)
 * It can use explicit coordinates (e.g: lon/lat) or implicit (WMTS coordinates)
 */

var CARDINAL = {
    WEST: 0,
    EAST: 1,
    SOUTH: 2,
    NORTH: 3
};

/**
 * @class      Extent
 * @param      {string}          crs     projection crs (ex: 'EPSG:4326')
 * @param      {(Array|object|Number)}  values  west, east, south and north values
 */
function Extent(crs) {
    var _this = this;

    for (var _len = arguments.length, values = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        values[_key - 1] = arguments[_key];
    }

    this._crs = crs;

    if (this.isTiledCrs()) {
        if (values.length == 3) {
            this.zoom = values[0];
            this.row = values[1];
            this.col = values[2];

            if (this.zoom < 0) {
                throw new Error('invlid WTMS values ' + values);
            }
        } else {
            throw new Error('Unsupported constructor args \'' + values + '\'');
        }
    } else if (values.length === 2 && values[0] instanceof _Coordinates2.default && values[1] instanceof _Coordinates2.default) {
        this._values = new Float64Array(4);
        this._values[CARDINAL.WEST] = values[0]._values[0];
        this._values[CARDINAL.EAST] = values[1]._values[0];
        this._values[CARDINAL.SOUTH] = values[0]._values[1];
        this._values[CARDINAL.NORTH] = values[1]._values[1];
    } else if (values.length == 1 && values[0].west != undefined) {
        this._values = new Float64Array(4);
        this._values[CARDINAL.WEST] = values[0].west;
        this._values[CARDINAL.EAST] = values[0].east;
        this._values[CARDINAL.SOUTH] = values[0].south;
        this._values[CARDINAL.NORTH] = values[0].north;
    } else if (values.length == 4) {
        this._values = new Float64Array(4);
        (0, _keys2.default)(CARDINAL).forEach(function (key) {
            var cardinal = CARDINAL[key];
            _this._values[cardinal] = values[cardinal];
        });
    } else {
        throw new Error('Unsupported constructor args \'' + values + '\'');
    }
}

Extent.prototype.clone = function () {
    if (this.isTiledCrs()) {
        return new Extent(this._crs, this.zoom, this.row, this.col);
    } else {
        var result = new (Function.prototype.bind.apply(Extent, [null].concat([this._crs], (0, _toConsumableArray3.default)(this._values))))();
        return result;
    }
};

Extent.prototype.isTiledCrs = function () {
    return this._crs.indexOf('WMTS:') == 0 || this._crs == 'TMS';
};

// EPSG:3857
// WGS84 bounds [-180.0, -85.06, 180.0, 85.06] (https://epsg.io/3857)
// Warning, some tiled source don't exactly match the same bound
// It should be taken into account
var coord = new _Coordinates2.default('EPSG:4326', 180, 85.06);
coord.as('EPSG:3857', coord);
// Get bound dimension in 'EPSG:3857'
var sizeX = coord._values[0] * 2;
var sizeY = coord._values[1] * 2;

Extent.prototype.as = function (crs) {
    (0, _Coordinates.assertCrsIsValid)(crs);

    if (this.isTiledCrs()) {
        if (this._crs == 'WMTS:PM' || this._crs == 'TMS') {
            // Convert this to the requested crs by using 4326 as an intermediate state.
            var nbCol = Math.pow(2, this.zoom);
            var nbRow = nbCol;
            var sizeRow = 1.0 / nbRow;
            // convert row PM to Y PM
            var Yn = 1 - sizeRow * (nbRow - this.row);
            var Ys = Yn + sizeRow;

            // convert to EPSG:3857
            if (crs == 'EPSG:3857') {
                var west = (0.5 - sizeRow * (nbCol - this.col)) * sizeX;

                return new Extent('EPSG:3857', { west: west, east: west + sizeRow * sizeX, south: (0.5 - Ys) * sizeY, north: (0.5 - Yn) * sizeY }).as(crs);
            } else {
                var size = 360 / nbCol;
                // convert Y PM to latitude EPSG:4326 degree
                var _north = _Projection2.default.YToWGS84(Yn);
                var _south = _Projection2.default.YToWGS84(Ys);
                // convert column PM to longitude EPSG:4326 degree
                var _west = 180 - size * (nbCol - this.col);


                var extent = Extent('EPSG:4326', { west: _west, east: _west + size, south: _south, north: _north });
                if (crs == 'EPSG:4326') {
                    return extent;
                } else {
                    // convert in new crs
                    return extent.as(crs);
                }
            }
        } else if (this._crs == 'WMTS:WGS84G' && crs == 'EPSG:4326') {
            var _nbRow = Math.pow(2, this.zoom);
            var _size = 180 / _nbRow;
            var _north2 = _size * (_nbRow - this.row) - 90;
            var _south2 = _size * (_nbRow - (this.row + 1)) - 90;
            var _west2 = 180 - _size * (2 * _nbRow - this.col);
            var _east2 = 180 - _size * (2 * _nbRow - (this.col + 1));

            return new Extent(crs, { west: _west2, east: _east2, south: _south2, north: _north2 });
        } else {
            throw new Error('Unsupported yet');
        }
    }

    if (this._crs != crs && !((0, _Coordinates.is4326)(this._crs) && (0, _Coordinates.is4326)(crs))) {
        // Compute min/max in x/y by projecting 8 cardinal points,
        // and then taking the min/max of each coordinates.
        var cardinals = [];
        var _c = this.center();
        cardinals.push(new _Coordinates2.default(this._crs, this.west(), this.north()));
        cardinals.push(new _Coordinates2.default(this._crs, _c._values[0], this.north()));
        cardinals.push(new _Coordinates2.default(this._crs, this.east(), this.north()));
        cardinals.push(new _Coordinates2.default(this._crs, this.east(), _c._values[1]));
        cardinals.push(new _Coordinates2.default(this._crs, this.east(), this.south()));
        cardinals.push(new _Coordinates2.default(this._crs, _c._values[0], this.south()));
        cardinals.push(new _Coordinates2.default(this._crs, this.west(), this.south()));
        cardinals.push(new _Coordinates2.default(this._crs, this.west(), _c._values[1]));

        var _north3 = -Infinity;
        var _south3 = Infinity;
        var _east3 = -Infinity;
        var _west3 = Infinity;
        // loop over the coordinates
        for (var i = 0; i < cardinals.length; i++) {
            // convert the coordinate.
            cardinals[i] = cardinals[i].as(crs);
            _north3 = Math.max(_north3, cardinals[i]._values[1]);
            _south3 = Math.min(_south3, cardinals[i]._values[1]);
            _east3 = Math.max(_east3, cardinals[i]._values[0]);
            _west3 = Math.min(_west3, cardinals[i]._values[0]);
        }
        return new Extent(crs, { north: _north3, south: _south3, east: _east3, west: _west3 });
    }

    return new Extent(crs, {
        west: this.west(),
        east: this.east(),
        north: this.north(),
        south: this.south()
    });
};

Extent.prototype.offsetToParent = function (other) {
    var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new THREE.Vector4();

    if (this.crs() != other.crs()) {
        throw new Error('unsupported mix');
    }
    if (this.isTiledCrs()) {
        var diffLevel = this.zoom - other.zoom;
        var diff = Math.pow(2, diffLevel);
        var invDiff = 1 / diff;

        var r = (this.row - this.row % diff) * invDiff;
        var _c2 = (this.col - this.col % diff) * invDiff;

        return target.set(this.col * invDiff - _c2, this.row * invDiff - r, invDiff, invDiff);
    }

    var dimension = {
        x: Math.abs(other.east() - other.west()),
        y: Math.abs(other.north() - other.south())
    };

    var originX = (this.west() - other.west()) / dimension.x;
    var originY = (other.north() - this.north()) / dimension.y;

    var scaleX = Math.abs(this.east() - this.west()) / dimension.x;

    var scaleY = Math.abs(this.north() - this.south()) / dimension.y;

    return target.set(originX, originY, scaleX, scaleY);
};

Extent.prototype.west = function () {
    return this._values[CARDINAL.WEST];
};

Extent.prototype.east = function () {
    return this._values[CARDINAL.EAST];
};

Extent.prototype.north = function () {
    return this._values[CARDINAL.NORTH];
};

Extent.prototype.south = function () {
    return this._values[CARDINAL.SOUTH];
};

Extent.prototype.crs = function () {
    return this._crs;
};

Extent.prototype.center = function (target) {
    if (this.isTiledCrs()) {
        throw new Error('Invalid operation for WMTS bbox');
    }
    var c = void 0;
    if (target) {
        _Coordinates2.default.call(target, this._crs, this._values[0], this._values[2]);
        c = target;
    } else {
        c = new _Coordinates2.default(this._crs, this._values[0], this._values[2]);
    }
    var dim = this.dimensions();
    c._values[0] += dim.x * 0.5;
    c._values[1] += dim.y * 0.5;
    return c;
};

Extent.prototype.dimensions = function (target) {
    target = target || { x: 0, y: 0 };
    target.x = Math.abs(this.east() - this.west());
    target.y = Math.abs(this.north() - this.south());
    return target;
};

/**
 * Return true if coord is inside the bounding box.
 *
 * @param {Coordinates} coord
 * @param {number} epsilon coord is inside the extent (+/- epsilon)
 * @return {boolean}
 */
Extent.prototype.isPointInside = function (coord) {
    var epsilon = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

    var c = this.crs() == coord.crs ? coord : coord.as(this.crs());
    // TODO this ignores altitude
    if ((0, _Coordinates.crsIsGeographic)(this.crs())) {
        return c.longitude() <= this.east() + epsilon && c.longitude() >= this.west() - epsilon && c.latitude() <= this.north() + epsilon && c.latitude() >= this.south() - epsilon;
    } else {
        return c.x() <= this.east() + epsilon && c.x() >= this.west() - epsilon && c.y() <= this.north() + epsilon && c.y() >= this.south() - epsilon;
    }
};

Extent.prototype.isInside = function (other, epsilon) {
    if (this.isTiledCrs()) {
        if (this.zoom == other.zoom) {
            return this.row == other.row && this.col == other.col;
        } else if (this.zoom < other.zoom) {
            return false;
        } else {
            var diffLevel = this.zoom - other.zoom;
            var diff = Math.pow(2, diffLevel);
            var invDiff = 1 / diff;

            var r = (this.row - this.row % diff) * invDiff;
            var _c3 = (this.col - this.col % diff) * invDiff;
            return r == other.row && _c3 == other.col;
        }
    } else {
        var o = other.as(this._crs);
        epsilon = epsilon == undefined ? (0, _Coordinates.reasonnableEpsilonForCRS)(this._crs) : epsilon;
        return this.east() - o.east() <= epsilon && o.west() - this.west() <= epsilon && this.north() - o.north() <= epsilon && o.south() - this.south() <= epsilon;
    }
};

Extent.prototype.offsetScale = function (bbox) {
    if (bbox.crs() != this.crs()) {
        throw new Error('unsupported offscale between 2 diff crs');
    }

    var dimension = {
        x: Math.abs(this.east() - this.west()),
        y: Math.abs(this.north() - this.south())
    };

    var originX = (bbox.west() - this.west()) / dimension.x;
    var originY = (bbox.north() - this.north()) / dimension.y;

    var scaleX = Math.abs(bbox.east() - bbox.west()) / dimension.x;
    var scaleY = Math.abs(bbox.north() - bbox.south()) / dimension.y;

    return new THREE.Vector4(originX, originY, scaleX, scaleY);
};

/**
 * @documentation: Return true if this bounding box intersect with the bouding box parameter
 * @param {type} bbox
 * @returns {Boolean}
 */
Extent.prototype.intersectsExtent = function (bbox) {
    var other = bbox.crs() == this.crs() ? bbox : bbox.as(this.crs());
    return !(this.west() >= other.east() || this.east() <= other.west() || this.south() >= other.north() || this.north() <= other.south());
};

/**
 * @documentation: Return the intersection of this extent with another one
 * @param {type} other
 * @returns {Boolean}
 */
Extent.prototype.intersect = function (other) {
    if (!this.intersectsExtent(other)) {
        return new Extent(this.crs(), 0, 0, 0, 0);
    }
    if (other.crs() != this.crs()) {
        other = other.as(this.crs());
    }
    var extent = new Extent(this.crs(), Math.max(this.west(), other.west()), Math.min(this.east(), other.east()), Math.max(this.south(), other.south()), Math.min(this.north(), other.north()));

    return extent;
};

Extent.prototype.set = function () {
    var _this2 = this;

    for (var _len2 = arguments.length, values = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        values[_key2] = arguments[_key2];
    }

    if (this.isTiledCrs()) {
        this.zoom = values[0];
        this.row = values[1];
        this.col = values[2];
    } else {
        (0, _keys2.default)(CARDINAL).forEach(function (key) {
            var cardinal = CARDINAL[key];
            _this2._values[cardinal] = values[cardinal];
        });
    }
    return this;
};

Extent.prototype.union = function (extent) {
    if (extent.crs() != this.crs()) {
        throw new Error('unsupported union between 2 diff crs');
    }
    var west = extent.west();
    if (west < this.west()) {
        this._values[CARDINAL.WEST] = west;
    }

    var east = extent.east();
    if (east > this.east()) {
        this._values[CARDINAL.EAST] = east;
    }

    var south = extent.south();
    if (south < this.south()) {
        this._values[CARDINAL.SOUTH] = south;
    }

    var north = extent.north();
    if (north > this.north()) {
        this._values[CARDINAL.NORTH] = north;
    }
};

/**
 * expandByPoint perfoms the minimal extension
 * for the point to belong to this Extent object
 * @param {Coordinates} coordinates  The coordinates to belong
 */
var c = new _Coordinates2.default('EPSG:4326', 0, 0, 0);
Extent.prototype.expandByPoint = function (coordinates) {
    var coords = coordinates.crs == this.crs() ? coordinates : coordinates.as(this.crs(), c);
    var we = coords._values[0];
    if (we < this.west()) {
        this._values[CARDINAL.WEST] = we;
    }
    if (we > this.east()) {
        this._values[CARDINAL.EAST] = we;
    }
    var sn = coords._values[1];
    if (sn < this.south()) {
        this._values[CARDINAL.SOUTH] = sn;
    }
    if (sn > this.north()) {
        this._values[CARDINAL.NORTH] = sn;
    }
};

Extent.fromBox3 = function (crs, box) {
    return new Extent(crs, {
        west: box.min.x,
        east: box.max.x,
        south: box.min.y,
        north: box.max.y
    });
};

Extent.prototype.extentParent = function (levelParent) {
    if (this.isTiledCrs()) {
        if (levelParent && levelParent < this.zoom) {
            var diffLevel = this.zoom - levelParent;
            var diff = Math.pow(2, diffLevel);
            var invDiff = 1 / diff;

            var r = (this.row - this.row % diff) * invDiff;
            var _c4 = (this.col - this.col % diff) * invDiff;

            return new Extent(this.crs(), levelParent, r, _c4);
        } else {
            return this;
        }
    }
};

Extent.prototype.toString = function () {
    var separator = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    if (this.isTiledCrs()) {
        return '' + this.zoom + separator + this.row + separator + this.col;
    } else {
        return '' + this.east() + separator + this.north() + separator + this.west() + separator + this.south();
    }
};

var center = new _Coordinates2.default('EPSG:4326', 0, 0, 0);
/**
 * Subdivide equally an extent from its center to return four extents:
 * north-west, north-east, south-west and south-east.
 *
 * @returns {Extent[]} An array containing the four sections of the extent. The
 * order of the sections is [NW, NE, SW, SE].
 */
Extent.prototype.subdivision = function () {
    this.center(center);

    var northWest = new Extent(this.crs(), this.west(), center._values[0], center._values[1], this.north());
    var northEast = new Extent(this.crs(), center._values[0], this.east(), center._values[1], this.north());
    var southWest = new Extent(this.crs(), this.west(), center._values[0], this.south(), center._values[1]);
    var southEast = new Extent(this.crs(), center._values[0], this.east(), this.south(), center._values[1]);

    return [northWest, northEast, southWest, southEast];
};

exports.default = Extent;