'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _get2 = require('babel-runtime/helpers/get');

var _get3 = _interopRequireDefault(_get2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _TiledGeometryLayer2 = require('../../../Layer/TiledGeometryLayer');

var _TiledGeometryLayer3 = _interopRequireDefault(_TiledGeometryLayer2);

var _Coordinates = require('../../Geographic/Coordinates');

var _Extent = require('../../Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _BuilderEllipsoidTile = require('./BuilderEllipsoidTile');

var _BuilderEllipsoidTile2 = _interopRequireDefault(_BuilderEllipsoidTile);

var _LayeredMaterialConstants = require('../../../Renderer/LayeredMaterialConstants');

var _OGCWebServiceHelper = require('../../../Provider/OGCWebServiceHelper');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// matrix to convert sphere to ellipsoid
var worldToScaledEllipsoid = new THREE.Matrix4();
// camera's position in worldToScaledEllipsoid system
var cameraPosition = new THREE.Vector3();
var magnitudeSquared = 0.0;

// vectors for operation purpose
var cullingVector = new THREE.Vector3();
var subdivisionVector = new THREE.Vector3();
var boundingSphereCenter = new THREE.Vector3();

// subdivison ratio
var subdivisionRatio = 0;

var GlobeLayer = function (_TiledGeometryLayer) {
    (0, _inherits3.default)(GlobeLayer, _TiledGeometryLayer);

    /**
     * A {@link TiledGeometryLayer} to use with a {@link GlobeView}. It has
     * specific method for updating and subdivising its grid.
     *
     * @constructor
     * @extends TiledGeometryLayer
     *
     * @param {string} id - The id of the layer, that should be unique. It is
     * not mandatory, but an error will be emitted if this layer is added a
     * {@link View} that already has a layer going by that id.
     * @param {THREE.Object3d} [object3d=THREE.Group] - The object3d used to
     * contain the geometry of the TiledGeometryLayer. It is usually a
     * <code>THREE.Group</code>, but it can be anything inheriting from a
     * <code>THREE.Object3d</code>.
     * @param {Object} [config] - Optional configuration, all elements in it
     * will be merged as is in the layer. For example, if the configuration
     * contains three elements <code>name, protocol, extent</code>, these
     * elements will be available using <code>layer.name</code> or something
     * else depending on the property name.
     * @param {number} [config.minSubdivisionLevel=2] - Minimum subdivision
     * level for this tiled layer.
     * @param {number} [config.maxSubdivisionLevel=18] - Maximum subdivision
     * level for this tiled layer.
     * @param {number} [config.sseSubdivisionThreshold=1] - Threshold level for
     * the SSE.
     * @param {number} [config.maxDeltaElevationLevel=4] - Maximum delta between
     * two elevations tile.
     *
     * @throws {Error} <code>object3d</code> must be a valid
     * <code>THREE.Object3d</code>.
     */
    function GlobeLayer(id, object3d) {
        var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        (0, _classCallCheck3.default)(this, GlobeLayer);

        // Configure tiles
        var schemeTile = [new _Extent2.default('EPSG:4326', -180, 0, -90, 90), new _Extent2.default('EPSG:4326', 0, 180, -90, 90)];
        var builder = new _BuilderEllipsoidTile2.default();

        var _this = (0, _possibleConstructorReturn3.default)(this, (GlobeLayer.__proto__ || (0, _getPrototypeOf2.default)(GlobeLayer)).call(this, id, object3d || new THREE.Group(), schemeTile, builder, config));

        _this.options.defaultPickingRadius = 5;
        _this.minSubdivisionLevel = _this.minSubdivisionLevel || 2.0;
        _this.maxSubdivisionLevel = _this.maxSubdivisionLevel || 18.0;
        _this.sseSubdivisionThreshold = _this.sseSubdivisionThreshold || 1.0;
        _this.maxDeltaElevation = _this.maxDeltaElevation || 4.0;

        _this.extent = _this.schemeTile[0].clone();

        for (var i = 1; i < _this.schemeTile.length; i++) {
            _this.extent.union(_this.schemeTile[i]);
        }

        // We're going to use the method described here:
        //    https://cesiumjs.org/2013/04/25/Horizon-culling/
        // This method assumes that the globe is a unit sphere at 0,0,0 so
        // we setup a world-to-scaled-ellipsoid matrix4
        worldToScaledEllipsoid.getInverse(_this.object3d.matrixWorld);
        worldToScaledEllipsoid.premultiply(new THREE.Matrix4().makeScale(1 / _Coordinates.ellipsoidSizes.x, 1 / _Coordinates.ellipsoidSizes.y, 1 / _Coordinates.ellipsoidSizes.z));

        subdivisionRatio = 1 / Math.pow(2, _this.maxDeltaElevationLevel);
        return _this;
    }

    (0, _createClass3.default)(GlobeLayer, [{
        key: 'preUpdate',
        value: function preUpdate(context, changeSources) {
            // pre-horizon culling
            cameraPosition.copy(context.camera.camera3D.position).applyMatrix4(worldToScaledEllipsoid);
            magnitudeSquared = cameraPosition.lengthSq() - 1.0;

            // pre-sse
            var canvasSize = context.view.mainLoop.gfxEngine.getWindowSize();
            var hypothenuse = canvasSize.length();
            var radAngle = context.view.camera.camera3D.fov * Math.PI / 180;
            var HYFOV = 2.0 * Math.atan(Math.tan(radAngle * 0.5) * hypothenuse / canvasSize.x);
            context.camera.preSSE = hypothenuse * (2.0 * Math.tan(HYFOV * 0.5));

            // Leaving the correct SSE below to be added later, as it is too heavy for now.
            // const FOV = THREE.Math.degToRad(context.camera.camera3D.fov);
            // context.camera.preSSE = context.camera.height / (2.0 * Math.tan(FOV * 0.5));

            return (0, _get3.default)(GlobeLayer.prototype.__proto__ || (0, _getPrototypeOf2.default)(GlobeLayer.prototype), 'preUpdate', this).call(this, context, changeSources);
        }

        // eslint-disable-next-line class-methods-use-this

    }, {
        key: 'countColorLayersTextures',
        value: function countColorLayersTextures() {
            var occupancy = 0;

            for (var _len = arguments.length, layers = Array(_len), _key = 0; _key < _len; _key++) {
                layers[_key] = arguments[_key];
            }

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)(layers), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var layer = _step.value;

                    var projection = layer.projection || layer.source.projection;
                    // 'EPSG:3857' occupies the maximum 3 textures on tiles
                    // 'EPSG:4326' occupies 1 textures on tile
                    occupancy += projection == 'EPSG:3857' ? 3 : 1;
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            return occupancy;
        }

        // eslint-disable-next-line

    }, {
        key: 'culling',
        value: function culling(node, camera) {
            if (!camera.isBox3Visible(node.obb.box3D, node.obb.matrixWorld)) {
                return true;
            }

            if (node.level < this.minSubdivisionLevel) {
                return false;
            }

            var points = node.obb.topPointsWorld;

            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = (0, _getIterator3.default)(points), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var point = _step2.value;

                    // see https://cesiumjs.org/2013/04/25/Horizon-culling/
                    cullingVector.copy(point);
                    cullingVector.applyMatrix4(worldToScaledEllipsoid).sub(cameraPosition);

                    var vtMagnitudeSquared = cullingVector.lengthSq();
                    var dot = -cullingVector.dot(cameraPosition);
                    var isOccluded = magnitudeSquared < dot && magnitudeSquared < dot * dot / vtMagnitudeSquared;

                    if (!isOccluded) {
                        return false;
                    }
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }

            return true;
        }

        /**
         * Test the subdvision of a node, compared to this layer.
         *
         * @param {Object} context - The context of the update; see the {@link
         * MainLoop} for more informations.
         * @param {GlobeLayer} layer - This layer, parameter to be removed.
         * @param {TileMesh} node - The node to test.
         *
         * @return {boolean} - True if the node is subdivisable, otherwise false.
         */

    }, {
        key: 'subdivision',
        value: function subdivision(context, layer, node) {
            if (node.level < this.minSubdivisionLevel) {
                return true;
            }

            if (this.maxSubdivisionLevel <= node.level) {
                return false;
            }

            // Prevent to subdivise the node if the current elevation level
            // we must avoid a tile, with level 20, inherits a level 3 elevation texture.
            // The induced geometric error is much too large and distorts the SSE
            var currentTexture = node.material.textures[_LayeredMaterialConstants.l_ELEVATION][0];
            if (currentTexture.extent) {
                var offsetScale = node.material.offsetScale[_LayeredMaterialConstants.l_ELEVATION][0];
                var ratio = offsetScale.z;
                // ratio is node size / texture size
                if (ratio < subdivisionRatio) {
                    return false;
                }
            }

            subdivisionVector.setFromMatrixScale(node.matrixWorld);
            boundingSphereCenter.copy(node.boundingSphere.center).applyMatrix4(node.matrixWorld);
            var distance = Math.max(0.0, context.camera.camera3D.position.distanceTo(boundingSphereCenter) - node.boundingSphere.radius * subdivisionVector.x);

            // TODO: node.geometricError is computed using a hardcoded 18 level
            // The computation of node.geometricError is surely false
            var sse = context.camera.preSSE * (node.geometricError * subdivisionVector.x) / distance;

            return this.sseSubdivisionThreshold < sse;
        }
    }, {
        key: 'computeTileZoomFromDistanceCamera',
        value: function computeTileZoomFromDistanceCamera(distance, camera) {
            var preSinus = _OGCWebServiceHelper.SIZE_TEXTURE_TILE * (this.sseSubdivisionThreshold * 0.5) / camera.preSSE / _Coordinates.ellipsoidSizes.x;

            var sinus = distance * preSinus;
            var zoom = Math.log(Math.PI / (2.0 * Math.asin(sinus))) / Math.log(2);

            var delta = Math.PI / Math.pow(2, zoom);
            var circleChord = 2.0 * _Coordinates.ellipsoidSizes.x * Math.sin(delta * 0.5);


            // adjust with bounding sphere rayon
            sinus = (distance - circleChord * 0.5) * preSinus;
            zoom = Math.log(Math.PI / (2.0 * Math.asin(sinus))) / Math.log(2);

            return isNaN(zoom) ? 0 : Math.round(zoom);
        }
    }, {
        key: 'computeDistanceCameraFromTileZoom',
        value: function computeDistanceCameraFromTileZoom(zoom, camera) {
            var delta = Math.PI / Math.pow(2, zoom);
            var circleChord = 2.0 * _Coordinates.ellipsoidSizes.x * Math.sin(delta * 0.5);
            var radius = circleChord * 0.5;


            return camera.preSSE * (radius / _OGCWebServiceHelper.SIZE_TEXTURE_TILE) / (this.sseSubdivisionThreshold * 0.5) + radius;
        }
    }]);
    return GlobeLayer;
}(_TiledGeometryLayer3.default);

exports.default = GlobeLayer;