'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _TiledGeometryLayer2 = require('../../../Layer/TiledGeometryLayer');

var _TiledGeometryLayer3 = _interopRequireDefault(_TiledGeometryLayer2);

var _Extent = require('../../Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _PanoramaTileBuilder = require('./PanoramaTileBuilder');

var _PanoramaTileBuilder2 = _interopRequireDefault(_PanoramaTileBuilder);

var _Constants = require('./Constants');

var _Constants2 = _interopRequireDefault(_Constants);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var textureSize = new THREE.Vector2(512, 256);
var center = new THREE.Vector3();

var PanoramaLayer = function (_TiledGeometryLayer) {
    (0, _inherits3.default)(PanoramaLayer, _TiledGeometryLayer);

    /**
     * A {@link TiledGeometryLayer} to use with a {@link PanoramaView}. It has
     * specific method for updating and subdivising its grid.
     *
     * @constructor
     * @extends TiledGeometryLayer
     *
     * @param {string} id - The id of the layer, that should be unique. It is
     * not mandatory, but an error will be emitted if this layer is added a
     * {@link View} that already has a layer going by that id.
     * @param {Coordinates} coordinates - The coordinates of the origin of the
     * panorama.
     * @param {number} type - The type of projection for the panorama: 1 for a
     * cylindrical projection, 2 for a spherical one.
     * @param {Object} [config] - Optional configuration, all elements in it
     * will be merged as is in the layer. For example, if the configuration
     * contains three elements <code>name, protocol, extent</code>, these
     * elements will be available using <code>layer.name</code> or something
     * else depending on the property name.
     * @param {THREE.Object3d} [config.object3d=THREE.Group] - The object3d used to
     * contain the geometry of the TiledGeometryLayer. It is usually a
     * <code>THREE.Group</code>, but it can be anything inheriting from a
     * <code>THREE.Object3d</code>.
     * @param {number} [config.maxSubdivisionLevel=10] - Maximum subdivision
     * level for this tiled layer.
     * @param {number} [config.ratio=1] - Ratio for building the panorama
     * sphere.
     *
     * @throws {Error} <code>object3d</code> must be a valid
     * <code>THREE.Object3d</code>.
     */
    function PanoramaLayer(id, coordinates, type, config) {
        (0, _classCallCheck3.default)(this, PanoramaLayer);

        var schemeTile = void 0;
        if (type === _Constants2.default.SPHERICAL) {
            // equirectangular -> spherical geometry
            schemeTile = [new _Extent2.default('EPSG:4326', {
                west: -180,
                east: 0,
                north: 90,
                south: -90
            }), new _Extent2.default('EPSG:4326', {
                west: 0,
                east: 180,
                north: 90,
                south: -90
            })];
        } else if (type === _Constants2.default.CYLINDRICAL) {
            // cylindrical geometry
            schemeTile = [new _Extent2.default('EPSG:4326', {
                west: -180,
                east: -90,
                north: 90,
                south: -90
            }), new _Extent2.default('EPSG:4326', {
                west: -90,
                east: 0,
                north: 90,
                south: -90
            }), new _Extent2.default('EPSG:4326', {
                west: 0,
                east: 90,
                north: 90,
                south: -90
            }), new _Extent2.default('EPSG:4326', {
                west: 90,
                east: 180,
                north: 90,
                south: -90
            })];
        } else {
            throw new Error('Unsupported panorama projection type ' + type + '.\n                Only ProjectionType.SPHERICAL and ProjectionType.CYLINDRICAL are supported');
        }
        var builder = new _PanoramaTileBuilder2.default(type, config.ratio || 1);

        var _this = (0, _possibleConstructorReturn3.default)(this, (PanoramaLayer.__proto__ || (0, _getPrototypeOf2.default)(PanoramaLayer)).call(this, id, config.object3d || new THREE.Group(), schemeTile, builder, config));

        coordinates.xyz(_this.object3d.position);
        _this.object3d.quaternion.setFromUnitVectors(new THREE.Vector3(0, 0, 1), coordinates.geodesicNormal);
        _this.object3d.updateMatrixWorld(true);

        // FIXME: add CRS = '0' support
        _this.extent = new _Extent2.default('EPSG:4326', {
            west: -180,
            east: 180,
            north: 90,
            south: -90
        });

        _this.disableSkirt = true;

        _this.options.segments = 8;
        _this.options.quality = 0.5;
        return _this;
    }

    // eslint-disable-next-line


    (0, _createClass3.default)(PanoramaLayer, [{
        key: 'culling',
        value: function culling(node, camera) {
            return !camera.isBox3Visible(node.obb.box3D, node.obb.matrixWorld);
        }

        /**
         * Test the subdvision of a node, compared to this layer.
         *
         * @param {Object} context - The context of the update; see the {@link
         * MainLoop} for more informations.
         * @param {PlanarLayer} layer - This layer, parameter to be removed.
         * @param {TileMesh} node - The node to test.
         *
         * @return {boolean} - True if the node is subdivisable, otherwise false.
         */

    }, {
        key: 'subdivision',
        value: function subdivision(context, layer, node) {
            if (node.level < 1) {
                return true;
            }

            var maxLevel = this.maxSubdivisionLevel || 10;

            if (maxLevel <= node.level) {
                return false;
            }

            var obb = node.obb;

            obb.updateMatrixWorld();
            var onScreen = context.camera.box3SizeOnScreen(obb.box3D, obb.matrixWorld);

            onScreen.min.z = 0;
            onScreen.max.z = 0;
            onScreen.getCenter(center);

            // give a small boost to central tiles
            var boost = 1 + Math.max(0, 1 - center.length());

            var dim = {
                x: 0.5 * (onScreen.max.x - onScreen.min.x) * context.camera.width,
                y: 0.5 * (onScreen.max.y - onScreen.min.y) * context.camera.height
            };

            var quality = layer.options.quality || 1.0;
            return boost * dim.x * quality >= textureSize.x && boost * dim.y * quality >= textureSize.y;
        }
    }]);
    return PanoramaLayer;
}(_TiledGeometryLayer3.default);

exports.default = PanoramaLayer;