'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

exports.createPlanarLayer = createPlanarLayer;

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _View = require('../View');

var _View2 = _interopRequireDefault(_View);

var _MainLoop = require('../MainLoop');

var _RendererConstant = require('../../Renderer/RendererConstant');

var _RendererConstant2 = _interopRequireDefault(_RendererConstant);

var _CameraUtils = require('../../utils/CameraUtils');

var _CameraUtils2 = _interopRequireDefault(_CameraUtils);

var _PlanarLayer = require('./Planar/PlanarLayer');

var _PlanarLayer2 = _interopRequireDefault(_PlanarLayer);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createPlanarLayer(id, extent, options) {
    console.warn('createPlanarLayer is deprecated, use the PlanarLayer class instead.');
    return new _PlanarLayer2.default(id, extent, options.object3d, options);
}

function PlanarView(viewerDiv, extent) {
    var _this = this;

    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    THREE.Object3D.DefaultUp.set(0, 0, 1);

    // Setup View
    _View2.default.call(this, extent.crs(), viewerDiv, options);

    // Configure camera
    var dim = extent.dimensions();
    var max = Math.max(dim.x, dim.y);
    var camera3D = this.camera.camera3D;
    camera3D.near = 0.1;
    camera3D.far = 2 * max;
    this.camera.camera3D.updateProjectionMatrix();

    var tileLayer = new _PlanarLayer2.default('planar', extent, options.object3d, options);

    this.addLayer(tileLayer);

    var p = { coord: extent.center(), range: max, tilt: 20, heading: 0 };
    _CameraUtils2.default.transformCameraToLookAtTarget(this, camera3D, p);

    this._renderState = _RendererConstant2.default.FINAL;
    this._fullSizeDepthBuffer = null;
    this.addFrameRequester(_MainLoop.MAIN_LOOP_EVENTS.BEFORE_RENDER, function () {
        if (_this._fullSizeDepthBuffer != null) {
            // clean depth buffer
            _this._fullSizeDepthBuffer = null;
        }
    });

    this.tileLayer = tileLayer;
}

PlanarView.prototype = (0, _create2.default)(_View2.default.prototype);
PlanarView.prototype.constructor = PlanarView;

PlanarView.prototype.addLayer = function (layer) {
    return _View2.default.prototype.addLayer.call(this, layer, this.tileLayer);
};

PlanarView.prototype.selectNodeAt = function (mouse) {
    var picked = this.tileLayer.pickObjectsAt(this, mouse);
    var selectedId = picked.length ? picked[0].object.id : undefined;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(this.tileLayer.level0Nodes), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var n = _step.value;

            n.traverse(function (node) {
                // only take of selectable nodes
                if (node.setSelected) {
                    node.setSelected(node.id === selectedId);

                    if (node.id === selectedId) {
                        // eslint-disable-next-line no-console
                        console.info(node);
                    }
                }
            });
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    this.notifyChange();
};

exports.default = PlanarView;