'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _Layer2 = require('./Layer');

var _Layer3 = _interopRequireDefault(_Layer2);

var _LayeredMaterialNodeProcessing = require('../Process/LayeredMaterialNodeProcessing');

var _textureConverter = require('../Parser/textureConverter');

var _textureConverter2 = _interopRequireDefault(_textureConverter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ElevationLayer = function (_Layer) {
    (0, _inherits3.default)(ElevationLayer, _Layer);

    /**
     * A simple layer, managing an elevation texture to add some reliefs on the
     * plane or globe view for example.
     *
     * @constructor
     * @extends Layer
     *
     * @param {string} id - The id of the layer, that should be unique. It is
     * not mandatory, but an error will be emitted if this layer is added a
     * {@link View} that already has a layer going by that id.
     * @param {Object} [config] - Optional configuration, all elements in it
     * will be merged as is in the layer. For example, if the configuration
     * contains three elements <code>name, protocol, extent</code>, these
     * elements will be available using <code>layer.name</code> or something
     * else depending on the property name.
     * @param {WMTSSource|WMSSource|WFSSource|TMSSource|FileSource} [config.source] data source
     *
     * @example
     * // Create an ElevationLayer
     * const elevation = new ElevationLayer('IGN_MNT', {
     *      source: {
     *          url: 'http://server.geo/wmts/SERVICE=WMTS&TILEMATRIX=%TILEMATRIX&TILEROW=%ROW&TILECOL=%COL',
     *          protocol: 'wmts',
     *          format: 'image/x-bil;bits=32',
     *      },
     * });
     *
     * // Add the layer
     * view.addLayer(elevation);
     *
     * @example
     * // Add and create an ElevationLayer
     * view.addLayer({
     *     id: 'IGN_MNT',
     *     type: 'elevation',
     *     source: {
     *          url: 'http://server.geo/wmts/SERVICE=WMTS&TILEMATRIX=%TILEMATRIX&TILEROW=%ROW&TILECOL=%COL',
     *          protocol: 'wmts',
     *          format: 'image/x-bil;bits=32',
     *     },
     * });
     */
    function ElevationLayer(id) {
        var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        (0, _classCallCheck3.default)(this, ElevationLayer);
        return (0, _possibleConstructorReturn3.default)(this, (ElevationLayer.__proto__ || (0, _getPrototypeOf2.default)(ElevationLayer)).call(this, id, 'elevation', config));
    }

    (0, _createClass3.default)(ElevationLayer, [{
        key: 'update',
        value: function update(context, layer, node, parent) {
            return (0, _LayeredMaterialNodeProcessing.updateLayeredMaterialNodeElevation)(context, this, node, parent);
        }
    }, {
        key: 'convert',
        value: function convert(data, extentDestination) {
            return _textureConverter2.default.convert(data, extentDestination, this);
        }
    }]);
    return ElevationLayer;
}(_Layer3.default);

exports.default = ElevationLayer;