'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _Layer2 = require('./Layer');

var _Layer3 = _interopRequireDefault(_Layer2);

var _Picking = require('../Core/Picking');

var _Picking2 = _interopRequireDefault(_Picking);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Fires when the opacity of the layer has changed.
 * @event GeometryLayer#opacity-property-changed
 */

var GeometryLayer = function (_Layer) {
    (0, _inherits3.default)(GeometryLayer, _Layer);

    /**
     * A layer usually managing a geometry to display on a view. For example, it
     * can be a layer of buildings extruded from a a WFS stream.
     *
     * @constructor
     * @extends Layer
     *
     * @param {string} id - The id of the layer, that should be unique. It is
     * not mandatory, but an error will be emitted if this layer is added a
     * {@link View} that already has a layer going by that id.
     * @param {THREE.Object3d} object3d - The object3d used to contain the
     * geometry of the GeometryLayer. It is usually a <code>THREE.Group</code>,
     * but it can be anything inheriting from a <code>THREE.Object3d</code>.
     * @param {Object} [config] - Optional configuration, all elements in it
     * will be merged as is in the layer. For example, if the configuration
     * contains three elements <code>name, protocol, extent</code>, these
     * elements will be available using <code>layer.name</code> or something
     * else depending on the property name.
     * @param {WFSSource|FileSource} [config.source] data source
     *
     * @throws {Error} <code>object3d</code> must be a valid
     * <code>THREE.Object3d</code>.
     *
     * @example
     * // Create a GeometryLayer
     * const geometry = new GeometryLayer('buildings', {
     *      source: {
     *          url: 'http://server.geo/wfs?',
     *          protocol: 'wfs',
     *          format: 'application/json'
     *      },
     * });
     *
     * // Add the layer
     * view.addLayer(geometry);
     *
     * @example
     * // Add and create a GeometryLayer
     * view.addLayer({
     *     id: 'buildings',
     *     type: 'geometry',
     *      source: {
     *          url: 'http://server.geo/wfs?',
     *          protocol: 'wfs',
     *          format: 'application/json'
     *      },
     * });
     */
    function GeometryLayer(id, object3d) {
        var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        (0, _classCallCheck3.default)(this, GeometryLayer);

        var _this = (0, _possibleConstructorReturn3.default)(this, (GeometryLayer.__proto__ || (0, _getPrototypeOf2.default)(GeometryLayer)).call(this, id, 'geometry', config));

        if (!object3d || !object3d.isObject3D) {
            throw new Error('Missing/Invalid object3d parameter (must be a\n                three.js Object3D instance)');
        }

        if (object3d.type === 'Group' && object3d.name === '') {
            object3d.name = id;
        }

        Object.defineProperty(_this, 'object3d', {
            value: object3d,
            writable: false,
            configurable: true
        });

        _this.defineLayerProperty('opacity', 1.0, function () {
            var root = _this.parent ? _this.parent.object3d : _this.object3d;
            root.traverse(function (object) {
                if (object.layer == _this) {
                    _this.changeOpacity(object);
                } else if (object.content && object.content.layer == _this) {
                    object.content.traverse(_this.changeOpacity);
                }
            });
        });

        _this.defineLayerProperty('wireframe', false, function () {
            var root = _this.parent ? _this.parent.object3d : _this.object3d;
            root.traverse(function (object) {
                if (object.layer == _this && object.material) {
                    object.material.wireframe = _this.wireframe;
                } else if (object.content && object.content.layer == _this) {
                    object.content.traverse(function (o) {
                        if (o.material) {
                            o.material.wireframe = _this.wireframe;
                        }
                    });
                }
            });
        });

        _this.attachedLayers = [];
        _this.visible = true;

        // Attached layers expect to receive the visual representation of a
        // layer (= THREE object with a material).  So if a layer's update
        // function don't process this kind of object, the layer must provide a
        // getObjectToUpdateForAttachedLayers function that returns the correct
        // object to update for attached layer.
        // See 3dtilesProvider or PointCloudProvider for examples.
        // eslint-disable-next-line arrow-body-style
        _this.getObjectToUpdateForAttachedLayers = function (obj) {
            if (obj.parent && obj.material) {
                return {
                    element: obj,
                    parent: obj.parent
                };
            }
        };
        return _this;
    }

    // Placeholder
    // eslint-disable-next-line


    (0, _createClass3.default)(GeometryLayer, [{
        key: 'postUpdate',
        value: function postUpdate() {}

        // Placeholder
        // eslint-disable-next-line

    }, {
        key: 'culling',
        value: function culling() {
            return true;
        }

        /**
         * Attach another layer to this one. Layers attached to a GeometryLayer will
         * be available in <code>geometryLayer.attachedLayers</code>.
         *
         * @param {Layer} layer - The layer to attach, that must have an
         * <code>update</code> method.
         */

    }, {
        key: 'attach',
        value: function attach(layer) {
            if (!layer.update) {
                throw new Error('Missing \'update\' function -> can\'t attach layer\n                ' + layer.id);
            }
            this.attachedLayers.push(layer);
            // To traverse GeometryLayer object3d attached
            layer.parent = this;
        }

        /**
         * Detach a layer attached to this one. See {@link attach} to learn how to
         * attach a layer.
         *
         * @param {Layer} layer - The layer to detach.
         *
         * @return {boolean} Confirmation of the detachment of the layer.
         */

    }, {
        key: 'detach',
        value: function detach(layer) {
            var count = this.attachedLayers.length;
            this.attachedLayers = this.attachedLayers.filter(function (attached) {
                return attached.id != layer.id;
            });
            layer.parent = undefined;
            return this.attachedLayers.length < count;
        }

        /**
         * Picking method for this layer. It uses the {@link Picking#pickObjectsAt}
         * method.
         *
         * @param {View} view - The view instance.
         * @param {Object} coordinates - The coordinates to pick in the view. It
         * should have at least <code>x</code> and <code>y</code> properties.
         * @param {number} radius - Radius of the picking circle.
         *
         * @return {Array} An array containing all targets picked under the
         * specified coordinates.
         */

    }, {
        key: 'pickObjectsAt',
        value: function pickObjectsAt(view, coordinates) {
            var radius = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.options.defaultPickingRadius;

            return _Picking2.default.pickObjectsAt(view, coordinates, radius, this.object3d);
        }

        /**
         * Change the opacity of an object, according to the value of the
         * <code>opacity</code> property of this layer.
         *
         * @param {Object} object - The object to change the opacity from. It is
         * usually a <code>THREE.Object3d</code> or an implementation of it.
         */

    }, {
        key: 'changeOpacity',
        value: function changeOpacity(object) {
            if (object.material) {
                // != undefined: we want the test to pass if opacity is 0
                if (object.material.opacity != undefined) {
                    object.material.transparent = this.opacity < 1.0;
                    object.material.opacity = this.opacity;
                }
                if (object.material.uniforms && object.material.uniforms.opacity != undefined) {
                    object.material.transparent = this.opacity < 1.0;
                    object.material.uniforms.opacity.value = this.opacity;
                }
            }
        }
    }]);
    return GeometryLayer;
}(_Layer3.default);

exports.default = GeometryLayer;