'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _GeometryLayer2 = require('./GeometryLayer');

var _GeometryLayer3 = _interopRequireDefault(_GeometryLayer2);

var _Picking = require('../Core/Picking');

var _Picking2 = _interopRequireDefault(_Picking);

var _convertToTile = require('../Parser/convertToTile');

var _convertToTile2 = _interopRequireDefault(_convertToTile);

var _CancelledCommandException = require('../Core/Scheduler/CancelledCommandException');

var _CancelledCommandException2 = _interopRequireDefault(_CancelledCommandException);

var _ObjectRemovalHelper = require('../Process/ObjectRemovalHelper');

var _ObjectRemovalHelper2 = _interopRequireDefault(_ObjectRemovalHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TiledGeometryLayer = function (_GeometryLayer) {
    (0, _inherits3.default)(TiledGeometryLayer, _GeometryLayer);

    /**
     * A layer extending the {@link GeometryLayer}, but with a tiling notion.
     *
     * @constructor
     * @extends GeometryLayer
     *
     * @param {string} id - The id of the layer, that should be unique. It is
     * not mandatory, but an error will be emitted if this layer is added a
     * {@link View} that already has a layer going by that id.
     * @param {THREE.Object3d} object3d - The object3d used to contain the
     * geometry of the TiledGeometryLayer. It is usually a
     * <code>THREE.Group</code>, but it can be anything inheriting from a
     * <code>THREE.Object3d</code>.
     * @param {Array} schemeTile - extents Array of root tiles
     * @param {Object} builder - builder geometry object
     * @param {Object} [config] - Optional configuration, all elements in it
     * will be merged as is in the layer. For example, if the configuration
     * contains three elements <code>name, protocol, extent</code>, these
     * elements will be available using <code>layer.name</code> or something
     * else depending on the property name.
     *
     * @throws {Error} <code>object3d</code> must be a valid
     * <code>THREE.Object3d</code>.
     */
    function TiledGeometryLayer(id, object3d, schemeTile, builder, config) {
        (0, _classCallCheck3.default)(this, TiledGeometryLayer);

        var _this = (0, _possibleConstructorReturn3.default)(this, (TiledGeometryLayer.__proto__ || (0, _getPrototypeOf2.default)(TiledGeometryLayer)).call(this, id, object3d, config));

        _this.protocol = 'tile';
        _this.lighting = {
            enable: false,
            position: { x: -0.5, y: 0.0, z: 1.0 }
        };

        _this.schemeTile = schemeTile;
        _this.builder = builder;

        if (!_this.schemeTile) {
            throw new Error('Cannot init tiled layer without schemeTile for layer ' + _this.id);
        }

        if (!_this.builder) {
            throw new Error('Cannot init tiled layer without builder for layer ' + _this.id);
        }

        _this.level0Nodes = [];
        var promises = [];

        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = (0, _getIterator3.default)(_this.schemeTile), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var root = _step.value;

                promises.push(_this.convert(undefined, root));
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        _promise2.default.all(promises).then(function (level0s) {
            _this.level0Nodes = level0s;
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = (0, _getIterator3.default)(level0s), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var level0 = _step2.value;

                    _this.object3d.add(level0);
                    level0.updateMatrixWorld();
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        });
        return _this;
    }

    /**
     * Picking method for this layer. It uses the {@link Picking#pickTilesAt}
     * method.
     *
     * @param {View} view - The view instance.
     * @param {Object} coordinates - The coordinates to pick in the view. It
     * should have at least <code>x</code> and <code>y</code> properties.
     * @param {number} radius - Radius of the picking circle.
     *
     * @return {Array} An array containing all targets picked under the
     * specified coordinates.
     */


    (0, _createClass3.default)(TiledGeometryLayer, [{
        key: 'pickObjectsAt',
        value: function pickObjectsAt(view, coordinates) {
            var radius = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.options.defaultPickingRadius;

            return _Picking2.default.pickTilesAt(view, coordinates, radius, this);
        }

        /**
         * Does pre-update work on the context:
         * <ul>
         *  <li>update the <code>colorLayers</code> and
         *  <code>elevationLayers</code></li>
         *  <li>update the <code>maxElevationLevel</code></li>
         * </ul>
         *
         * Once this work is done, it returns a list of nodes to update. Depending
         * on the origin of <code>sources</code>, it can return a few things:
         * <ul>
         *  <li>if <code>sources</code> is empty, it returns the first node of the
         *  layer (stored as <code>level0Nodes</code>), which will trigger the
         *  update of the whole tree</li>
         *  <li>if the update is triggered by a camera move, the whole tree is
         *  returned too</li>
         *  <li>if <code>source.layer</code> is this layer, it means that
         *  <code>source</code> is a node; a common ancestor will be found if there
         *  are multiple sources, with the default common ancestor being the first
         *  source itself</li>
         *  <li>else it returns the whole tree</li>
         * </ul>
         *
         * @param {Object} context - The context of the update; see the {@link
         * MainLoop} for more informations.
         * @param {Set<GeometryLayer|TileMesh>} sources - A list of sources to
         * generate a list of nodes to update.
         *
         * @return {TileMesh[]} The array of nodes to update.
         */

    }, {
        key: 'preUpdate',
        value: function preUpdate(context, sources) {
            var _this2 = this;

            if (sources.has(undefined) || sources.size == 0) {
                return this.level0Nodes;
            }

            context.colorLayers = context.view.getLayers(function (l, a) {
                return a && a.id == _this2.id && l.type == 'color';
            });
            context.elevationLayers = context.view.getLayers(function (l, a) {
                return a && a.id == _this2.id && l.type == 'elevation';
            });

            context.maxElevationLevel = -1;
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = (0, _getIterator3.default)(context.elevationLayers), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var e = _step3.value;

                    context.maxElevationLevel = Math.max(e.source.zoom.max, context.maxElevationLevel);
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }

            if (context.maxElevationLevel == -1) {
                context.maxElevationLevel = Infinity;
            }

            var commonAncestor = void 0;
            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = (0, _getIterator3.default)(sources.values()), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    var source = _step4.value;

                    if (source.isCamera) {
                        // if the change is caused by a camera move, no need to bother
                        // to find common ancestor: we need to update the whole tree:
                        // some invisible tiles may now be visible
                        return this.level0Nodes;
                    }
                    if (source.layer === this) {
                        if (!commonAncestor) {
                            commonAncestor = source;
                        } else {
                            commonAncestor = source.findCommonAncestor(commonAncestor);
                            if (!commonAncestor) {
                                return this.level0Nodes;
                            }
                        }
                        if (commonAncestor.material == null) {
                            commonAncestor = undefined;
                        }
                    }
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }

            if (commonAncestor) {
                return [commonAncestor];
            } else {
                return this.level0Nodes;
            }
        }

        /**
         * Update a node of this layer. The node will not be updated if:
         * <ul>
         *  <li>it does not have a parent, then it is removed</li>
         *  <li>its parent is being subdivided</li>
         *  <li>is not visible in the camera</li>
         * </ul>
         *
         * @param {Object} context - The context of the update; see the {@link
         * MainLoop} for more informations.
         * @param {Layer} layer - Parameter to be removed once all update methods
         * have been aligned.
         * @param {TileMesh} node - The node to update.
         *
         * @returns {Object}
         */

    }, {
        key: 'update',
        value: function update(context, layer, node) {
            var _this3 = this;

            if (!node.parent) {
                return _ObjectRemovalHelper2.default.removeChildrenAndCleanup(this, node);
            }
            // early exit if parent' subdivision is in progress
            if (node.parent.pendingSubdivision) {
                node.visible = false;
                node.setDisplayed(false);
                return undefined;
            }

            // do proper culling
            node.visible = !this.culling(node, context.camera);

            if (node.visible) {
                var requestChildrenUpdate = false;

                if (node.pendingSubdivision || TiledGeometryLayer.hasEnoughTexturesToSubdivide(context, node) && this.subdivision(context, this, node)) {
                    this.subdivideNode(context, node);
                    // display iff children aren't ready
                    node.setDisplayed(node.pendingSubdivision);
                    requestChildrenUpdate = true;
                } else {
                    node.setDisplayed(true);
                }

                if (node.material.visible) {
                    // update uniforms
                    if (context.view.fogDistance != undefined) {
                        node.setFog(context.view.fogDistance);
                    }

                    if (!requestChildrenUpdate) {
                        return _ObjectRemovalHelper2.default.removeChildren(this, node);
                    }
                }

                return requestChildrenUpdate ? node.children.filter(function (n) {
                    return n.layer == _this3;
                }) : undefined;
            }

            node.setDisplayed(false);
            return _ObjectRemovalHelper2.default.removeChildren(this, node);
        }
    }, {
        key: 'convert',
        value: function convert(requester, extent) {
            return _convertToTile2.default.convert(requester, extent, this);
        }

        // eslint-disable-next-line class-methods-use-this

    }, {
        key: 'countColorLayersTextures',
        value: function countColorLayersTextures() {
            return arguments.length;
        }

        /**
         * Tell if a node has enough elevation or color textures to subdivide.
         * Subdivision is prevented if:
         * <ul>
         *  <li>the node is covered by at least one elevation layer and if the node
         *  doesn't have an elevation texture yet</li>
         *  <li>a color texture is missing</li>
         * </ul>
         *
         * @param {Object} context - The context of the update; see the {@link
         * MainLoop} for more informations.
         * @param {TileMesh} node - The node to subdivide.
         *
         * @returns {boolean} False if the node can not be subdivided, true
         * otherwise.
         */

    }, {
        key: 'subdivideNode',


        /**
         * Subdivides a node of this layer. If the node is currently in the process
         * of subdivision, it will not do anything here. The subdivision of a node
         * will occur in four part, to create a quadtree. The extent of the node
         * will be divided in four parts: north-west, north-east, south-west and
         * south-east. Once all four nodes are created, they will be added to the
         * current node and the view of the context will be notified of this change.
         *
         * @param {Object} context - The context of the update; see the {@link
         * MainLoop} for more informations.
         * @param {TileMesh} node - The node to subdivide.
         */
        value: function subdivideNode(context, node) {
            var _this4 = this;

            if (!node.pendingSubdivision && !node.children.some(function (n) {
                return n.layer == _this4;
            })) {
                var extents = node.extent.subdivision();
                // TODO: pendingSubdivision mechanism is fragile, get rid of it
                node.pendingSubdivision = true;

                var command = {
                    /* mandatory */
                    view: context.view,
                    requester: node,
                    layer: this,
                    priority: 10000,
                    /* specific params */
                    extentsSource: extents,
                    redraw: false
                };

                context.scheduler.execute(command).then(function (children) {
                    var _iteratorNormalCompletion5 = true;
                    var _didIteratorError5 = false;
                    var _iteratorError5 = undefined;

                    try {
                        for (var _iterator5 = (0, _getIterator3.default)(children), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
                            var child = _step5.value;

                            node.add(child);
                            child.updateMatrixWorld(true);
                        }
                    } catch (err) {
                        _didIteratorError5 = true;
                        _iteratorError5 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion5 && _iterator5.return) {
                                _iterator5.return();
                            }
                        } finally {
                            if (_didIteratorError5) {
                                throw _iteratorError5;
                            }
                        }
                    }

                    node.pendingSubdivision = false;
                    context.view.notifyChange(node, false);
                }, function (err) {
                    node.pendingSubdivision = false;
                    if (!(err instanceof _CancelledCommandException2.default)) {
                        throw new Error(err);
                    }
                });
            }
        }
    }], [{
        key: 'hasEnoughTexturesToSubdivide',
        value: function hasEnoughTexturesToSubdivide(context, node) {
            var _iteratorNormalCompletion6 = true;
            var _didIteratorError6 = false;
            var _iteratorError6 = undefined;

            try {
                for (var _iterator6 = (0, _getIterator3.default)(context.elevationLayers), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
                    var e = _step6.value;

                    var extents = node.getCoordsForSource(e.source);
                    if (!e.frozen && e.ready && e.source.extentsInsideLimit(extents) && !node.material.isElevationLayerLoaded()) {
                        // no stop subdivision in the case of a loading error
                        if (node.layerUpdateState[e.id] && node.layerUpdateState[e.id].inError()) {
                            continue;
                        }
                        return false;
                    }
                }
            } catch (err) {
                _didIteratorError6 = true;
                _iteratorError6 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion6 && _iterator6.return) {
                        _iterator6.return();
                    }
                } finally {
                    if (_didIteratorError6) {
                        throw _iteratorError6;
                    }
                }
            }

            var _iteratorNormalCompletion7 = true;
            var _didIteratorError7 = false;
            var _iteratorError7 = undefined;

            try {
                for (var _iterator7 = (0, _getIterator3.default)(context.colorLayers), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                    var c = _step7.value;

                    if (c.frozen || !c.visible || !c.ready) {
                        continue;
                    }
                    // no stop subdivision in the case of a loading error
                    if (node.layerUpdateState[c.id] && node.layerUpdateState[c.id].inError()) {
                        continue;
                    }
                    var _extents = node.getCoordsForSource(c.source);
                    if (c.source.extentsInsideLimit(_extents) && !node.material.isColorLayerLoaded(c)) {
                        return false;
                    }
                }
            } catch (err) {
                _didIteratorError7 = true;
                _iteratorError7 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion7 && _iterator7.return) {
                        _iterator7.return();
                    }
                } finally {
                    if (_didIteratorError7) {
                        throw _iteratorError7;
                    }
                }
            }

            return true;
        }
    }]);
    return TiledGeometryLayer;
}(_GeometryLayer3.default);

exports.default = TiledGeometryLayer;