'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _Feature2Texture = require('../Renderer/ThreeExtended/Feature2Texture');

var _Feature2Texture2 = _interopRequireDefault(_Feature2Texture);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var textureLayer = function (texture) {
    texture.generateMipmaps = false;
    texture.magFilter = THREE.LinearFilter;
    texture.minFilter = THREE.LinearFilter;
    return texture;
};

function textureColorLayer(texture, transparent) {
    texture.anisotropy = 16;
    texture.premultiplyAlpha = transparent;
    return textureLayer(texture);
}

exports.default = {
    convert: function convert(data, extentDestination, layer) {
        var texture = void 0;
        if (data.isFeature) {
            var backgroundColor = layer.backgroundLayer && layer.backgroundLayer.paint ? new THREE.Color(layer.backgroundLayer.paint['background-color']) : undefined;

            var extentTexture = extentDestination.as(layer.projection);
            texture = _Feature2Texture2.default.createTextureFromFeature(data, extentTexture, 256, layer.style, backgroundColor);
            texture.parsedData = data;
            texture.coords = extentDestination;
        } else if (data.isTexture) {
            texture = data;
        } else {
            throw new Error('Data type is not supported to convert into texture');
        }

        if (layer.type === 'color') {
            return textureColorLayer(texture, layer.transparent);
        } else if (layer.type === 'elevation') {
            if (texture.flipY) {
                // DataTexture default to false, so make sure other Texture types
                // do the same (eg image texture)
                // See UV construction for more details
                texture.flipY = false;
            }
            return textureLayer(texture);
        }
    }
};