'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

exports.updateLayeredMaterialNodeImagery = updateLayeredMaterialNodeImagery;
exports.updateLayeredMaterialNodeElevation = updateLayeredMaterialNodeElevation;

var _LayeredMaterialConstants = require('../Renderer/LayeredMaterialConstants');

var _LayerUpdateStrategy = require('../Layer/LayerUpdateStrategy');

var _LayerUpdateState = require('../Layer/LayerUpdateState');

var _LayerUpdateState2 = _interopRequireDefault(_LayerUpdateState);

var _Layer = require('../Layer/Layer');

var _CancelledCommandException = require('../Core/Scheduler/CancelledCommandException');

var _CancelledCommandException2 = _interopRequireDefault(_CancelledCommandException);

var _OGCWebServiceHelper = require('../Provider/OGCWebServiceHelper');

var _XbilParser = require('../Parser/XbilParser');

var _XbilParser2 = _interopRequireDefault(_XbilParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// max retry loading before changing the status to definitiveError
var MAX_RETRY = 4;

function getSourceExtent(node, extent, targetLevel, source) {
    if (source && source.getSourceExtents) {
        return source.getSourceExtents(extent).extent;
    } else if (extent.isTiledCrs()) {
        return extent.extentParent(targetLevel);
    } else {
        var parent = node.findAncestorFromLevel(targetLevel);
        if (parent.extent) {
            // Needed to initNodeElevationTextureFromParent, insertSignificantValuesFromParent,
            // isColorLayerDownscaled
            // Must be removed
            parent.extent.zoom = parent.level;
        }
        return parent.extent;
    }
}

function initNodeImageryTexturesFromParent(node, parent, layer) {
    if (parent.material && parent.material.isColorLayerLoaded(layer)) {
        var coords = node.getCoordsForSource(layer.source);
        var offsetTextures = node.material.getLayerTextureOffset(layer.id);

        var textureIndex = offsetTextures;
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = (0, _getIterator3.default)(coords), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var c = _step.value;
                var _iteratorNormalCompletion2 = true;
                var _didIteratorError2 = false;
                var _iteratorError2 = undefined;

                try {
                    for (var _iterator2 = (0, _getIterator3.default)(parent.material.getLayerTextures(layer)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                        var texture = _step2.value;

                        if (c.isInside(texture.coords)) {
                            var result = c.offsetToParent(texture.coords);
                            node.material.textures[_LayeredMaterialConstants.l_COLOR][textureIndex] = texture;
                            node.material.offsetScale[_LayeredMaterialConstants.l_COLOR][textureIndex] = result;
                            textureIndex++;
                            break;
                        }
                    }
                } catch (err) {
                    _didIteratorError2 = true;
                    _iteratorError2 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion2 && _iterator2.return) {
                            _iterator2.return();
                        }
                    } finally {
                        if (_didIteratorError2) {
                            throw _iteratorError2;
                        }
                    }
                }
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        var index = node.material.indexOfColorLayer(layer.id);
        node.material.layerTexturesCount[index] = coords.length;
        node.material.loadedTexturesCount[_LayeredMaterialConstants.l_COLOR] += coords.length;
    }
}

function initNodeElevationTextureFromParent(node, parent, layer) {
    // Inherit parent's elevation texture. Note that contrary to color layers the elevation level of the
    // node might not be EMPTY_TEXTURE_ZOOM in this init function. That's because we can have
    // multiple elevation layers (thus multiple calls to initNodeElevationTextureFromParent) but a given
    // node can only use 1 elevation texture
    if (parent.material && parent.material.getElevationLayerLevel() > node.material.getElevationLayerLevel()) {
        var coords = node.getCoordsForSource(layer.source);

        var texture = parent.material.textures[_LayeredMaterialConstants.l_ELEVATION][0];
        var pitch = coords[0].offsetToParent(parent.material.textures[_LayeredMaterialConstants.l_ELEVATION][0].coords);
        var elevation = {
            texture: texture,
            pitch: pitch
        };

        // If the texture resolution has a poor precision for this node, we don't
        // extract min-max from the texture (too few information), we instead chose
        // to use parent's min-max.
        var useMinMaxFromParent = node.level - texture.coords.zoom > 6;
        if (!useMinMaxFromParent) {
            var _computeMinMaxElevati = (0, _XbilParser2.default)(texture.image.data, _OGCWebServiceHelper.SIZE_TEXTURE_TILE, _OGCWebServiceHelper.SIZE_TEXTURE_TILE, pitch),
                min = _computeMinMaxElevati.min,
                max = _computeMinMaxElevati.max;

            elevation.min = min;
            elevation.max = max;
        }

        node.setTextureElevation(layer, elevation, pitch);
    }
}

function getIndiceWithPitch(i, pitch, w) {
    // normalized
    var currentY = Math.floor(i / w) / w; // normalized

    // Return corresponding indice in parent tile using pitch
    var newX = pitch.x + i % w / w * pitch.z;
    var newY = pitch.y + currentY * pitch.w;
    var newIndice = Math.floor(newY * w) * w + Math.floor(newX * w);
    return newIndice;
}

function insertSignificantValuesFromParent(texture, node, parent, layer) {
    if (parent.material && parent.material.getElevationLayerLevel() > _LayeredMaterialConstants.EMPTY_TEXTURE_ZOOM) {
        var coords = node.getCoordsForSource(layer.source);
        var textureParent = parent.material.textures[_LayeredMaterialConstants.l_ELEVATION][0];
        var pitch = coords[0].offsetToParent(parent.material.textures[_LayeredMaterialConstants.l_ELEVATION][0].coords);
        var tData = texture.image.data;
        var l = tData.length;

        for (var i = 0; i < l; ++i) {
            if (tData[i] === layer.noDataValue) {
                tData[i] = textureParent.image.data[getIndiceWithPitch(i, pitch, 256)];
            }
        }
    }
}

function nodeCommandQueuePriorityFunction(node) {
    // We know that 'node' is visible because commands can only be
    // issued for visible nodes.

    // TODO: need priorization of displayed nodes
    if (node.isDisplayed()) {
        // Then prefer displayed() node over non-displayed one
        return 100;
    } else {
        return 10;
    }
}

function refinementCommandCancellationFn(cmd) {
    if (!cmd.requester.parent || !cmd.requester.material) {
        return true;
    }
    if (cmd.force) {
        return false;
    }

    // Cancel the command if the tile already has a better texture.
    // This is only needed for elevation layers, because we may have several
    // concurrent layers but we can only use one texture.
    if (cmd.layer.type == 'elevation' && cmd.targetLevel <= cmd.requester.material.getElevationLayerLevel()) {
        return true;
    }

    return !cmd.requester.isDisplayed();
}

function checkNodeElevationTextureValidity(texture, noDataValue) {
    // We check if the elevation texture has some significant values through corners
    var tData = texture.image.data;
    var l = tData.length;
    return tData[0] > noDataValue && tData[l - 1] > noDataValue && tData[Math.sqrt(l) - 1] > noDataValue && tData[l - Math.sqrt(l)] > noDataValue;
}

function updateLayeredMaterialNodeImagery(context, layer, node, parent) {
    var material = node.material;
    var extentsDestination = node.getCoordsForSource(layer.source);

    // Initialisation
    if (node.layerUpdateState[layer.id] === undefined) {
        node.layerUpdateState[layer.id] = new _LayerUpdateState2.default();

        if (!layer.source.extentsInsideLimit(extentsDestination)) {
            // we also need to check that tile's parent doesn't have a texture for this layer,
            // because even if this tile is outside of the layer, it could inherit it's
            // parent texture
            if (!(!layer.noTextureParentOutsideLimit && parent && parent.material && parent.material.indexOfColorLayer && parent.material.indexOfColorLayer(layer.id) >= 0)) {
                node.layerUpdateState[layer.id].noMoreUpdatePossible();
                return;
            }
        }

        if (material.indexOfColorLayer(layer.id) === -1) {
            material.pushLayer(layer, node.getCoordsForSource(layer.source));
            var imageryLayers = context.view.getLayers(function (l) {
                return l.type === 'color';
            });
            var sequence = _Layer.ImageryLayers.getColorLayersIdOrderedBySequence(imageryLayers);
            material.setSequence(sequence);

            initNodeImageryTexturesFromParent(node, parent, layer);
        }

        // Proposed new process, two separate processes:
        //      * FIRST PASS: initNodeXXXFromParent and get out of the function
        //      * SECOND PASS: Fetch best texture

        // The two-step allows you to filter out unnecessary requests
        // Indeed in the second pass, their state (not visible or not displayed) can block them to fetch
        var minLevel = layer.source.zoom.min;
        if (node.material.getColorLayerLevelById(layer.id) >= minLevel) {
            context.view.notifyChange(node, false);
            return;
        }
    }

    // Node is hidden, no need to update it
    if (!node.isDisplayed()) {
        return;
    }

    // TODO: move this to defineLayerProperty() declaration
    // to avoid mixing layer's network updates and layer's params
    // Update material parameters
    material.setLayerVisibility(layer, layer.visible);
    material.setLayerOpacity(layer, layer.opacity);

    var ts = Date.now();
    // An update is pending / or impossible -> abort
    if (!layer.visible || !node.layerUpdateState[layer.id].canTryUpdate(ts)) {
        return;
    }

    if (layer.source.canTileTextureBeImproved) {
        // if the layer has a custom method -> use it
        if (!layer.source.canTileTextureBeImproved(node.extent, material.getLayerTextures(layer)[0])) {
            node.layerUpdateState[layer.id].noMoreUpdatePossible();
            return;
        }
    } else if (!material.isColorLayerDownscaled(layer, node.getZoomForLayer(layer))) {
        // default decision method
        node.layerUpdateState[layer.id].noMoreUpdatePossible();
        return;
    }

    // is fetching data from this layer disabled?
    if (layer.frozen) {
        return;
    }

    var failureParams = node.layerUpdateState[layer.id].failureParams;
    var currentLevel = node.material.getColorLayerLevelById(layer.id);
    var nodeLevel = extentsDestination[0].zoom || node.level;
    var targetLevel = (0, _LayerUpdateStrategy.chooseNextLevelToFetch)(layer.updateStrategy.type, node, nodeLevel, currentLevel, layer, failureParams);
    if (targetLevel <= currentLevel) {
        return;
    }

    // Get equivalent of extent destination in source
    var extentsSource = [];
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
        for (var _iterator3 = (0, _getIterator3.default)(extentsDestination), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var extentDestination = _step3.value;

            var extentSource = getSourceExtent(node, extentDestination, targetLevel, layer.source);
            if (extentSource && !layer.source.extentInsideLimit(extentSource)) {
                // Retry extentInsideLimit because you must check with the targetLevel
                // if the first test extentInsideLimit returns that it is out of limits
                // and the node inherits from its parent, then it'll still make a command to fetch texture.
                node.layerUpdateState[layer.id].noMoreUpdatePossible();
                return;
            }
            extentsSource.push(extentSource);
        }
    } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
            }
        } finally {
            if (_didIteratorError3) {
                throw _iteratorError3;
            }
        }
    }

    var parsedData = node.material.getLayerTextures(layer).map(function (t) {
        return t.parsedData;
    });
    node.layerUpdateState[layer.id].newTry();
    var command = {
        /* mandatory */
        view: context.view,
        layer: layer,
        extentsSource: extentsSource,
        extentsDestination: extentsDestination,
        parsedData: parsedData,
        requester: node,
        priority: nodeCommandQueuePriorityFunction(node),
        earlyDropFunction: refinementCommandCancellationFn
    };

    return context.scheduler.execute(command).then(function (result) {
        if (node.material === null) {
            return;
        }

        if (result) {
            var pitchs = extentsDestination.map(function (ext, i) {
                return ext.offsetToParent(extentsSource[i]);
            });
            node.material.setLayerTextures(layer, result, pitchs);
        }

        node.layerUpdateState[layer.id].success();

        return result;
    }, function (err) {
        if (err instanceof _CancelledCommandException2.default) {
            node.layerUpdateState[layer.id].success();
        } else {
            var definitiveError = node.layerUpdateState[layer.id].errorCount > MAX_RETRY;
            node.layerUpdateState[layer.id].failure(Date.now(), definitiveError, { targetLevel: targetLevel });
            if (!definitiveError) {
                window.setTimeout(function () {
                    context.view.notifyChange(node, false);
                }, node.layerUpdateState[layer.id].secondsUntilNextTry() * 1000);
            }
        }
    });
}

function updateLayeredMaterialNodeElevation(context, layer, node, parent) {
    // TODO: we need either
    //  - compound or exclusive layers
    //  - support for multiple elevation layers

    // Elevation is currently handled differently from color layers.
    // This is caused by a LayeredMaterial limitation: only 1 elevation texture
    // can be used (where a tile can have N textures x M layers)
    var material = node.material;
    var currentElevation = material.getElevationLayerLevel();

    // Init elevation layer, and inherit from parent if possible
    if (node.layerUpdateState[layer.id] === undefined) {
        node.layerUpdateState[layer.id] = new _LayerUpdateState2.default();
        initNodeElevationTextureFromParent(node, parent, layer);
        currentElevation = material.getElevationLayerLevel();
        var minLevel = layer.source.zoom.min;
        if (currentElevation >= minLevel) {
            context.view.notifyChange(node, false);
            return;
        }
    }

    // Try to update
    var ts = Date.now();

    // Possible conditions to *not* update the elevation texture
    if (layer.frozen || !node.isDisplayed() || !node.layerUpdateState[layer.id].canTryUpdate(ts)) {
        return;
    }

    // Does this tile needs a new texture?
    if (layer.canTileTextureBeImproved) {
        // if the layer has a custom method -> use it
        if (!layer.canTileTextureBeImproved(layer, node)) {
            node.layerUpdateState[layer.id].noMoreUpdatePossible();
            return;
        }
    }

    var extentsDestination = node.getCoordsForSource(layer.source);
    var targetLevel = (0, _LayerUpdateStrategy.chooseNextLevelToFetch)(layer.updateStrategy.type, node, extentsDestination[0].zoom, currentElevation, layer);

    if (targetLevel <= currentElevation) {
        node.layerUpdateState[layer.id].noMoreUpdatePossible();
        return _promise2.default.resolve();
    }

    var extentsSource = [];
    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
        for (var _iterator4 = (0, _getIterator3.default)(extentsDestination), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
            var nodeExtent = _step4.value;

            var extentSource = getSourceExtent(node, nodeExtent, targetLevel);
            if (extentSource && !layer.source.extentInsideLimit(extentSource)) {
                node.layerUpdateState[layer.id].noMoreUpdatePossible();
                return;
            }
            extentsSource.push(extentSource);
        }
    } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
            }
        } finally {
            if (_didIteratorError4) {
                throw _iteratorError4;
            }
        }
    }

    node.layerUpdateState[layer.id].newTry();

    var command = {
        /* mandatory */
        view: context.view,
        layer: layer,
        extentsSource: extentsSource,
        extentsDestination: extentsDestination,
        requester: node,
        priority: nodeCommandQueuePriorityFunction(node),
        earlyDropFunction: refinementCommandCancellationFn
    };

    return context.scheduler.execute(command).then(function (textures) {
        var terrain = { texture: textures[0] };

        // TODO: this check is maybe useless
        // see in dataSourceProvider
        if (node.material === null) {
            return;
        }

        // Do not apply the new texture if its level is < than the current one.
        // This is only needed for elevation layers, because we may have several
        // concurrent layers but we can only use one texture.
        if (targetLevel <= node.material.getElevationLayerLevel()) {
            node.layerUpdateState[layer.id].noMoreUpdatePossible();
            return;
        }

        node.layerUpdateState[layer.id].success();

        if (terrain.texture && terrain.texture.image.data && !checkNodeElevationTextureValidity(terrain.texture, layer.noDataValue)) {
            // Quick check to avoid using elevation texture with no data value
            // If we have no data values, we use value from the parent tile
            // We should later implement multi elevation layer to choose the one to use at each level
            insertSignificantValuesFromParent(terrain.texture, node, parent, layer);
        }

        if (terrain.texture && terrain.texture.image.data) {
            var _computeMinMaxElevati2 = (0, _XbilParser2.default)(terrain.texture.image.data),
                min = _computeMinMaxElevati2.min,
                max = _computeMinMaxElevati2.max;

            terrain.min = !min ? 0 : min;
            terrain.max = !max ? 0 : max;
        }
        node.setTextureElevation(layer, terrain, extentsDestination[0].offsetToParent(extentsSource[0]));
    }, function (err) {
        if (err instanceof _CancelledCommandException2.default) {
            node.layerUpdateState[layer.id].success();
        } else {
            var definitiveError = node.layerUpdateState[layer.id].errorCount > MAX_RETRY;
            node.layerUpdateState[layer.id].failure(Date.now(), definitiveError);
            if (!definitiveError) {
                window.setTimeout(function () {
                    context.view.notifyChange(node, false);
                }, node.layerUpdateState[layer.id].secondsUntilNextTry() * 1000);
            }
        }
    });
}