'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.supportedFetchers = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _GeoJsonParser = require('../Parser/GeoJsonParser');

var _GeoJsonParser2 = _interopRequireDefault(_GeoJsonParser);

var _VectorTileParser = require('../Parser/VectorTileParser');

var _VectorTileParser2 = _interopRequireDefault(_VectorTileParser);

var _Fetcher = require('./Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

var _Cache = require('../Core/Scheduler/Cache');

var _Cache2 = _interopRequireDefault(_Cache);

var _CancelledCommandException = require('../Core/Scheduler/CancelledCommandException');

var _CancelledCommandException2 = _interopRequireDefault(_CancelledCommandException);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var supportedFetchers = exports.supportedFetchers = new _map2.default([['image/png', _Fetcher2.default.texture], ['image/jpg', _Fetcher2.default.texture], ['image/jpeg', _Fetcher2.default.texture], ['image/x-bil;bits=32', _Fetcher2.default.textureFloat], ['geojson', _Fetcher2.default.json], ['application/json', _Fetcher2.default.json], ['application/json', _Fetcher2.default.json], ['application/x-protobuf;type=mapbox-vector', _Fetcher2.default.arrayBuffer]]);

function noParsingNeeded(data) {
    return data;
}

var supportedParsers = new _map2.default([['geojson', _GeoJsonParser2.default.parse], ['application/json', _GeoJsonParser2.default.parse], ['application/x-protobuf;type=mapbox-vector', _VectorTileParser2.default.parse], [true, noParsingNeeded]]);

function isValidData(data, extentDestination, validFn) {
    if (data && (!validFn || validFn(data, extentDestination))) {
        return data;
    }
}

function fetchData(url, format, networkOptions, extentSource) {
    var fetcher = supportedFetchers.get(format);
    if (fetcher) {
        return fetcher(url, networkOptions).then(function (d) {
            d.coords = extentSource;
            return d;
        });
    } else {
        throw new Error('Not supported format, not found fetcher in DataSourceProvider.supportedFetchers');
    }
}

function parseData(data, layer, extentDestination) {
    var type = data.isTexture || data.isFeature || layer.source.format;
    var options = {
        buildExtent: layer.type !== 'geometry',
        crsIn: layer.source.projection,
        crsOut: layer.projection,
        // TODO FIXME: error in filtering vector tile
        // filteringExtent: extentDestination.as(layer.projection),
        filteringExtent: layer.type === 'geometry' ? extentDestination : undefined,
        filter: layer.filter,
        origin: layer.source.origin,
        mergeFeatures: layer.mergeFeatures === undefined ? true : layer.mergeFeatures,
        withNormal: layer.type === 'geometry',
        withAltitude: layer.type === 'geometry'
    };
    return supportedParsers.get(type)(data, options);
}

var error = function (err, url, source) {
    source.handlingError(err, url);
    throw err;
};
function FetchAndConvertSourceData(url, layer, extentSource, extentDestination) {
    var source = layer.source;
    // Fetch data
    return fetchData(url, source.format, source.networkOptions, extentSource).then(function (fetchedData) {
        return (
            // Parse fetched data, it parses file to itowns's object
            parseData(fetchedData, layer, extentDestination)
        );
    }, function (err) {
        return error(err, url, source);
    }).then(function (parsedData) {
        return (
            // Convert parsed data, it converts itowns's object to THREE's object
            layer.convert(parsedData, extentDestination)
        );
    }, function (err) {
        return error(err, url, source);
    });
}

exports.default = {
    executeCommand: function executeCommand(command) {
        var promises = [];
        var layer = command.layer;
        var source = layer.source;
        var requester = command.requester;
        var extentsSource = command.extentsSource;
        var extentsDestination = command.extentsDestination || extentsSource;
        var parsedData = command.parsedData || [];

        // TODO: Find best place to cancel Command
        if (requester && !requester.material) {
            // request has been deleted
            return _promise2.default.reject(new _CancelledCommandException2.default(command));
        }

        for (var i = 0, max = extentsSource.length; i < max; i++) {
            var extSource = extentsSource[i];
            var extDest = extentsDestination[i];

            // If source, we must fetch and convert data
            // URL of the resource you want to fetch
            var url = source.urlFromExtent(extSource);

            // Already fetched and parsed data that can be used
            var validedParsedData = isValidData(parsedData[i], extDest, layer.isValidData) || source.parsedData;

            // Tag to Cache data
            var tag = validedParsedData ? url + ',' + extDest.toString(',') : url;

            // Get converted source data, in cache
            var convertedSourceData = _Cache2.default.get(tag);

            // If data isn't in cache
            if (!convertedSourceData) {
                if (validedParsedData) {
                    // Use parsed data
                    convertedSourceData = layer.convert(validedParsedData, extDest, layer);
                } else {
                    // Fetch and convert
                    convertedSourceData = FetchAndConvertSourceData(url, layer, extSource, extDest);
                }
                // Put converted data in cache
                _Cache2.default.set(tag, convertedSourceData, _Cache2.default.POLICIES.TEXTURE);
            }

            // Verify some command is resolved
            // See old WFSProvider : command.resolve(result)
            promises.push(convertedSourceData);
        }

        return _promise2.default.all(promises);
    }
};