'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _togeojson = require('togeojson');

var _togeojson2 = _interopRequireDefault(_togeojson);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _Feature2Texture = require('../Renderer/ThreeExtended/Feature2Texture');

var _Feature2Texture2 = _interopRequireDefault(_Feature2Texture);

var _GeoJsonParser = require('../Parser/GeoJsonParser');

var _GeoJsonParser2 = _interopRequireDefault(_GeoJsonParser);

var _Fetcher = require('./Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Class: RasterProvider
 * Description: Provides textures from a vector data
 */

function getExtentFromGpxFile(file) {
    var bound = file.getElementsByTagName('bounds')[0];
    if (bound) {
        var west = bound.getAttribute('minlon');
        var east = bound.getAttribute('maxlon');
        var south = bound.getAttribute('minlat');
        var north = bound.getAttribute('maxlat');
        return new _Extent2.default('EPSG:4326', west, east, south, north);
    }
    return new _Extent2.default('EPSG:4326', -180, 180, -90, 90);
}

function createTextureFromVector(tile, layer) {
    if (!tile.material) {
        return _promise2.default.resolve();
    }

    if (layer.type == 'color') {
        var coords = tile.extent;
        var result = { pitch: new THREE.Vector4(0, 0, 1, 1) };
        result.texture = _Feature2Texture2.default.createTextureFromFeature(layer.feature, tile.extent, 256, layer.style);
        result.texture.extent = tile.extent;
        result.texture.coords = coords;
        result.texture.coords.zoom = tile.level;

        if (layer.transparent) {
            result.texture.premultiplyAlpha = true;
        }
        return _promise2.default.resolve(result);
    } else {
        return _promise2.default.resolve();
    }
}

exports.default = {
    preprocessDataLayer: function preprocessDataLayer(layer, view, scheduler, parentLayer) {
        if (!layer.url) {
            throw new Error('layer.url is required');
        }

        // KML and GPX specifications all says that they should be in
        // EPSG:4326. We still support reprojection for them through this
        // configuration option
        layer.projection = layer.projection || 'EPSG:4326';
        var parentCrs = parentLayer.extent.crs();

        if (!(layer.extent instanceof _Extent2.default)) {
            layer.extent = new _Extent2.default(layer.projection, layer.extent).as(parentCrs);
        }

        if (!layer.options.zoom) {
            layer.options.zoom = { min: 5, max: 21 };
        }

        layer.style = layer.style || {};

        // Rasterization of data vector
        // It shouldn't use parent's texture outside its extent
        // Otherwise artefacts appear at the outer edge
        layer.noTextureParentOutsideLimit = true;

        return _Fetcher2.default.text(layer.url, layer.networkOptions).then(function (text) {
            var geojson = void 0;
            var trimmedText = text.trim();
            // We test the start of the string to choose a parser
            if (trimmedText.startsWith('<')) {
                // if it's an xml file, then it can be kml or gpx
                var parser = new DOMParser();
                var file = parser.parseFromString(text, 'application/xml');
                if (file.documentElement.tagName.toLowerCase() === 'kml') {
                    geojson = _togeojson2.default.kml(file);
                } else if (file.documentElement.tagName.toLowerCase() === 'gpx') {
                    geojson = _togeojson2.default.gpx(file);
                    layer.style.stroke = layer.style.stroke || 'red';
                    layer.extent = layer.extent.intersect(getExtentFromGpxFile(file).as(layer.extent.crs()));
                } else if (file.documentElement.tagName.toLowerCase() === 'parsererror') {
                    throw new Error('Error parsing XML document');
                } else {
                    throw new Error('Unsupported xml file, only valid KML and GPX are supported, but no <gpx> or <kml> tag found.', file);
                }
            } else if (trimmedText.startsWith('{') || trimmedText.startsWith('[')) {
                geojson = JSON.parse(text);
                if (geojson.type !== 'Feature' && geojson.type !== 'FeatureCollection') {
                    throw new Error('This json is not a GeoJSON');
                }
            } else {
                throw new Error('Unsupported file: only well-formed KML, GPX or GeoJSON are supported');
            }

            if (geojson) {
                var options = {
                    buildExtent: true,
                    crsIn: layer.projection,
                    crsOut: parentCrs,
                    filteringExtent: layer.extent
                };

                return _GeoJsonParser2.default.parse(geojson, options);
            }
        }).then(function (feature) {
            if (feature) {
                layer.feature = feature;
                layer.extent = layer.feature.extent || layer.feature.geometry.extent;
            }
        });
    },
    tileInsideLimit: function tileInsideLimit(tile, layer) {
        return tile.level >= layer.options.zoom.min && tile.level <= layer.options.zoom.max && layer.extent.intersectsExtent(tile.extent);
    },
    executeCommand: function executeCommand(command) {
        var layer = command.layer;
        var tile = command.requester;

        return createTextureFromVector(tile, layer);
    }
};