'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _flatbush = require('flatbush');

var _flatbush2 = _interopRequireDefault(_flatbush);

var _three = require('three');

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _OGCWebServiceHelper = require('./OGCWebServiceHelper');

var _OGCWebServiceHelper2 = _interopRequireDefault(_OGCWebServiceHelper);

var _Fetcher = require('./Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _selectImagesFromSpatialIndex(index, images, extent) {
    return index.search(extent.west(), extent.south(), extent.east(), extent.north()).map(function (i) {
        return images[i];
    });
}

// select the smallest image entirely covering the tile
function selectBestImageForExtent(layer, extent) {
    var candidates = _selectImagesFromSpatialIndex(layer._spatialIndex, layer.images, extent.as(layer.extent.crs()));

    var selection = void 0;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(candidates), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var entry = _step.value;

            if (extent.isInside(entry.extent)) {
                if (!selection) {
                    selection = entry;
                } else {
                    var d = selection.extent.dimensions();
                    var e = entry.extent.dimensions();
                    if (e.x <= d.x && e.y <= d.y) {
                        selection = entry;
                    }
                }
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return selection;
}

function buildUrl(layer, image) {
    return layer.url.href.substr(0, layer.url.href.lastIndexOf('/') + 1) + image;
}

function getTexture(tile, layer, targetLevel) {
    if (!tile.material) {
        return _promise2.default.resolve();
    }

    if (!layer.images) {
        return _promise2.default.reject();
    }

    var original = tile;
    if (targetLevel) {
        while (tile && tile.level > targetLevel) {
            tile = tile.parent;
        }
        if (!tile) {
            return _promise2.default.reject('Invalid targetLevel requested ' + targetLevel);
        }
    }

    var selection = selectBestImageForExtent(layer, tile.extent);

    if (!selection) {
        return _promise2.default.reject(new Error('No available image for tile ' + tile));
    }

    var fn = layer.format.indexOf('image/x-bil') === 0 ? _OGCWebServiceHelper2.default.getXBilTextureByUrl : _OGCWebServiceHelper2.default.getColorTextureByUrl;
    return fn(buildUrl(layer, selection.image), layer.networkOptions).then(function (texture) {
        // adjust pitch
        var result = {
            texture: texture,
            pitch: new _three.Vector4(0, 0, 1, 1)
        };

        result.texture.extent = selection.extent;
        result.texture.coords = selection.extent;
        if (!result.texture.coords.zoom || result.texture.coords.zoom > tile.level) {
            result.texture.coords.zoom = tile.level;
            result.texture.file = selection.image;
        }

        result.pitch = original.extent.offsetToParent(selection.extent);
        if (layer.transparent) {
            texture.premultiplyAlpha = true;
        }

        return result;
    });
}

/**
 * This provider uses no protocol but instead download static images directly.
 *
 * It uses as input 'image_filename: extent' values and then tries to find the best image
 * for a given tile using the extent property.
 */
exports.default = {
    preprocessDataLayer: function preprocessDataLayer(layer) {
        if (!layer.extent) {
            throw new Error('layer.extent is required');
        }

        if (!(layer.extent instanceof _Extent2.default)) {
            layer.extent = new (Function.prototype.bind.apply(_Extent2.default, [null].concat([layer.projection], (0, _toConsumableArray3.default)(layer.extent))))();
        }

        layer.canTileTextureBeImproved = this.canTileTextureBeImproved;
        layer.url = new URL(layer.url, window.location);
        return _Fetcher2.default.json(layer.url.href).then(function (metadata) {
            layer.images = [];
            // eslint-disable-next-line guard-for-in
            for (var image in metadata) {
                var extent = new (Function.prototype.bind.apply(_Extent2.default, [null].concat([layer.projection], (0, _toConsumableArray3.default)(metadata[image]))))();
                layer.images.push({
                    image: image,
                    extent: extent
                });
            }
            layer._spatialIndex = (0, _flatbush2.default)(layer.images.length);
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = (0, _getIterator3.default)(layer.images), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var _image = _step2.value;

                    layer._spatialIndex.add(_image.extent.west(), _image.extent.south(), _image.extent.east(), _image.extent.north());
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }

            layer._spatialIndex.finish();
        }).then(function () {
            if (!layer.format) {
                // fetch the first image to detect format
                if (layer.images.length) {
                    var url = buildUrl(layer, layer.images[0].image);
                    return fetch(url, layer.networkOptions).then(function (response) {
                        layer.format = response.headers.get('Content-type');
                        if (layer.format === 'application/octet-stream') {
                            layer.format = 'image/x-bil';
                        }
                        if (!layer.format) {
                            throw new Error(layer.name + ': could not detect layer format, please configure \'layer.format\'.');
                        }
                    });
                }
            }
        });
    },
    tileInsideLimit: function tileInsideLimit(tile, layer) {
        if (!layer.images) {
            return false;
        }

        return _selectImagesFromSpatialIndex(layer._spatialIndex, layer.images, tile.extent.as(layer.extent.crs())).length > 0;
    },
    canTileTextureBeImproved: function canTileTextureBeImproved(layer, tile) {
        if (!layer.images) {
            return false;
        }
        var s = selectBestImageForExtent(layer, tile.extent);
        if (!s) {
            return false;
        }
        var materialLayer = tile.material.getLayer(layer.id);
        var texture = materialLayer && materialLayer.textures[0];
        var file = texture && texture.file;
        return !file || file != s.image;
    },
    executeCommand: function executeCommand(command) {
        var tile = command.requester;
        var layer = command.layer;
        return getTexture(tile, layer, command.targetLevel);
    }
};