'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _OGCWebServiceHelper = require('./OGCWebServiceHelper');

var _OGCWebServiceHelper2 = _interopRequireDefault(_OGCWebServiceHelper);

var _URLBuilder = require('./URLBuilder');

var _URLBuilder2 = _interopRequireDefault(_URLBuilder);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _Coordinates = require('../Core/Geographic/Coordinates');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function preprocessDataLayer(layer) {
    if (!layer.extent) {
        // default to the full 3857 extent
        layer.extent = new _Extent2.default('EPSG:3857', -20037508.342789244, 20037508.342789244, -20037508.342789255, 20037508.342789244);
    }
    if (!(layer.extent instanceof _Extent2.default)) {
        if (!layer.projection) {
            throw new Error('Missing projection property for layer \'' + layer.id + '\'');
        }
        layer.extent = new (Function.prototype.bind.apply(_Extent2.default, [null].concat([layer.projection], (0, _toConsumableArray3.default)(layer.extent))))();
    }
    layer.origin = layer.origin || (layer.protocol == 'xyz' ? 'top' : 'bottom');
    if (!layer.options.zoom) {
        layer.options.zoom = {
            min: 0,
            max: 18
        };
    }
    layer.getCoords = function (node) {
        // Special globe case: use the P(seudo)M(ercator) coordinates
        if ((0, _Coordinates.is4326)(node.extent.crs()) && (this.extent.crs() == 'EPSG:3857' || (0, _Coordinates.is4326)(this.extent.crs()))) {
            _OGCWebServiceHelper2.default.computeTileMatrixSetCoordinates(node, 'PM');
            return node.wmtsCoords.PM;
        } else {
            return _OGCWebServiceHelper2.default.computeTMSCoordinates(node, this.extent, this.origin);
        }
    };
}

function executeCommand(command) {
    var layer = command.layer;
    var tile = command.requester;

    var promises = [];
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        var _loop = function () {
            var coordTMS = _step.value;

            var coordTMSParent = command.targetLevel < coordTMS.zoom ? _OGCWebServiceHelper2.default.WMTS_WGS84Parent(coordTMS, command.targetLevel) : undefined;

            var urld = _URLBuilder2.default.xyz(coordTMSParent || coordTMS, layer);

            promises.push(_OGCWebServiceHelper2.default.getColorTextureByUrl(urld, layer.networkOptions).then(function (texture) {
                var result = {};
                result.texture = texture;
                result.texture.coords = coordTMSParent || coordTMS;
                result.pitch = coordTMSParent ? coordTMS.offsetToParent(coordTMSParent) : new THREE.Vector4(0, 0, 1, 1);
                if (layer.transparent) {
                    texture.premultiplyAlpha = true;
                }
                return result;
            }));
        };

        for (var _iterator = (0, _getIterator3.default)(layer.getCoords(tile)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            _loop();
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return _promise2.default.all(promises);
}

function tileTextureCount(tile, layer) {
    return tileInsideLimit(tile, layer) ? 1 : 0;
}

function tileInsideLimit(tile, layer, targetLevel) {
    // assume 1 TMS texture per tile (ie: tile geometry CRS is the same as layer's CRS)
    var tmsCoord = layer.getCoords(tile)[0];

    if (targetLevel < tmsCoord.zoom) {
        tmsCoord = _OGCWebServiceHelper2.default.WMTS_WGS84Parent(tmsCoord, targetLevel);
    }

    return layer.options.zoom.min <= tmsCoord.zoom && tmsCoord.zoom <= layer.options.zoom.max;
}

exports.default = {
    preprocessDataLayer: preprocessDataLayer,
    executeCommand: executeCommand,
    tileTextureCount: tileTextureCount,
    tileInsideLimit: tileInsideLimit
};