'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _URLBuilder = require('./URLBuilder');

var _URLBuilder2 = _interopRequireDefault(_URLBuilder);

var _Fetcher = require('./Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

var _Cache = require('../Core/Scheduler/Cache');

var _Cache2 = _interopRequireDefault(_Cache);

var _GeoJsonParser = require('../Parser/GeoJsonParser');

var _GeoJsonParser2 = _interopRequireDefault(_GeoJsonParser);

var _Feature2Mesh = require('../Renderer/ThreeExtended/Feature2Mesh');

var _Feature2Mesh2 = _interopRequireDefault(_Feature2Mesh);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Generated On: 2016-03-5
 * Class: WFSProvider
 * Description: Provides data from a WFS stream
 */

function preprocessDataLayer(layer) {
    if (!layer.typeName) {
        throw new Error('layer.typeName is required.');
    }

    layer.format = layer.format || 'application/json';

    layer.crs = layer.projection || 'EPSG:4326';
    layer.version = layer.version || '2.0.2';
    layer.opacity = layer.opacity || 1;
    layer.wireframe = layer.wireframe || false;
    if (!(layer.extent instanceof _Extent2.default)) {
        layer.extent = new _Extent2.default(layer.projection, layer.extent);
    }
    layer.url = layer.url + 'SERVICE=WFS&REQUEST=GetFeature&typeName=' + layer.typeName + '&VERSION=' + layer.version + '&SRSNAME=' + layer.crs + '&outputFormat=' + layer.format + '&BBOX=%bbox,' + layer.crs;
}

function tileInsideLimit(tile, layer) {
    return (layer.level === undefined || tile.level === layer.level) && layer.extent.intersectsExtent(tile.extent);
}

function executeCommand(command) {
    var layer = command.layer;
    var tile = command.requester;
    var destinationCrs = command.view.referenceCrs;
    return getFeatures(destinationCrs, tile, layer, command).then(function (result) {
        return command.resolve(result);
    });
}

function assignLayer(object, layer) {
    if (object) {
        object.layer = layer;
        object.layers.set(layer.threejsLayer);
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = (0, _getIterator3.default)(object.children), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var c = _step.value;

                assignLayer(c, layer);
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        return object;
    }
}

function getFeatures(crs, tile, layer) {
    if (!layer.tileInsideLimit(tile, layer) || tile.material === null) {
        return _promise2.default.resolve();
    }

    var urld = _URLBuilder2.default.bbox(tile.extent.as(layer.crs), layer);

    layer.convert = layer.convert ? layer.convert : _Feature2Mesh2.default.convert({});

    return (_Cache2.default.get(urld) || _Cache2.default.set(urld, _Fetcher2.default.json(urld, layer.networkOptions))).then(function (geojson) {
        return _GeoJsonParser2.default.parse(geojson, { crsOut: crs, filteringExtent: tile.extent, filter: layer.filter });
    }, function (err) {
        // special handling for 400 errors, as it probably means the config is wrong
        if (err.response.status == 400) {
            return err.response.text().then(function (text) {
                var getCapUrl = layer.url + 'SERVICE=WFS&REQUEST=GetCapabilities&VERSION=' + layer.version;
                var xml = new DOMParser().parseFromString(text, 'application/xml');
                var errorElem = xml.querySelector('Exception');
                var errorCode = errorElem.getAttribute('exceptionCode');
                var errorMessage = errorElem.querySelector('ExceptionText').textContent;
                console.error('Layer ' + layer.name + ': bad request when fetching data. Server says: "' + errorCode + ': ' + errorMessage + '". \nReviewing ' + getCapUrl + ' may help.', err);
                throw err;
            });
        } else {
            console.error('Layer ' + layer.name + ': ' + err.response.status + ' error while trying to fetch WFS data. Url was ' + urld + '.', err);
            throw err;
        }
    }).then(function (feature) {
        return assignLayer(layer.convert(feature), layer);
    });
}

exports.default = {
    preprocessDataLayer: preprocessDataLayer,
    executeCommand: executeCommand,
    tileInsideLimit: tileInsideLimit
};