'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _OGCWebServiceHelper = require('./OGCWebServiceHelper');

var _OGCWebServiceHelper2 = _interopRequireDefault(_OGCWebServiceHelper);

var _URLBuilder = require('./URLBuilder');

var _URLBuilder2 = _interopRequireDefault(_URLBuilder);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Generated On: 2015-10-5
 * Class: WMSProvider
 * Description: Provides data from a WMS stream
 */

var supportedFormats = ['image/png', 'image/jpg', 'image/jpeg'];

function tileTextureCount(tile, layer) {
    return tile.extent.crs() == layer.projection ? 1 : layer.getCoords(tile).length;
}

function preprocessDataLayer(layer) {
    if (!layer.name) {
        throw new Error('layer.name is required.');
    }
    if (!layer.extent) {
        throw new Error('layer.extent is required');
    }
    if (!layer.projection) {
        throw new Error('layer.projection is required');
    }

    if (!(layer.extent instanceof _Extent2.default)) {
        layer.extent = new _Extent2.default(layer.projection, layer.extent);
    }

    if (!layer.options.zoom) {
        layer.options.zoom = { min: 0, max: 21 };
    }

    layer.format = layer.format || 'image/png';
    if (!supportedFormats.includes(layer.format)) {
        throw new Error('Layer ' + layer.name + ': unsupported format \'' + layer.format + '\', should be one of \'' + supportedFormats.join('\', \'') + '\'');
    }

    layer.width = layer.heightMapWidth || 256;
    layer.version = layer.version || '1.3.0';
    layer.style = layer.style || '';
    layer.transparent = layer.transparent || false;

    if (!layer.axisOrder) {
        // 4326 (lat/long) axis order depends on the WMS version used
        if (layer.projection == 'EPSG:4326') {
            // EPSG 4326 x = lat, long = y
            // version 1.1.0 long/lat while version 1.3.0 mandates xy (so lat,long)
            layer.axisOrder = layer.version === '1.1.0' ? 'wsen' : 'swne';
        } else {
            // xy,xy order
            layer.axisOrder = 'wsen';
        }
    }
    var crsPropName = 'SRS';
    if (layer.version === '1.3.0') {
        crsPropName = 'CRS';
    }

    layer.url = layer.url + '?SERVICE=WMS&REQUEST=GetMap&LAYERS=' + layer.name + '&VERSION=' + layer.version + '&STYLES=' + layer.style + '&FORMAT=' + layer.format + '&TRANSPARENT=' + layer.transparent + '&BBOX=%bbox' + ('&' + crsPropName + '=' + layer.projection + '&WIDTH=' + layer.width + '&HEIGHT=' + layer.width);

    layer.getCoords = function (node) {
        if (this.projection == node.extent.crs()) {
            return [node.extent];
        }
        if (this.projection != 'EPSG:3857') {
            throw new Error('unsupported projection wms for this viewer');
        }
        var tilematrixset = 'PM';
        _OGCWebServiceHelper2.default.computeTileMatrixSetCoordinates(node, tilematrixset);
        return node.wmtsCoords[tilematrixset];
    };
}

function tileInsideLimit(tile, layer) {
    return tile.level >= layer.options.zoom.min && tile.level <= layer.options.zoom.max && layer.extent.intersectsExtent(tile.extent);
}

function getColorTexture(tile, layer, targetLevel, tileCoords) {
    if (!tileInsideLimit(tile, layer)) {
        return _promise2.default.reject('Tile \'' + tile + '\' is outside layer bbox ' + layer.extent);
    }
    if (tile.material === null) {
        return _promise2.default.resolve();
    }

    var extent = tileCoords ? tileCoords.as(layer.projection) : tile.extent;
    // if no specific level requester, use tile.level
    if (targetLevel === undefined) {
        targetLevel = tile.level;
    } else if (!tileCoords) {
        var parentAtLevel = tile;
        while (parentAtLevel && parentAtLevel.level > targetLevel) {
            parentAtLevel = parentAtLevel.parent;
        }
        if (!parentAtLevel) {
            return _promise2.default.reject('Invalid targetLevel requested ' + targetLevel);
        }
        extent = parentAtLevel.extent;
        targetLevel = parentAtLevel.level;
    }

    var coords = extent.as(layer.projection);
    var urld = _URLBuilder2.default.bbox(coords, layer);
    var pitch = tileCoords ? new THREE.Vector4(0, 0, 1, 1) : tile.extent.offsetToParent(extent);
    var result = { pitch: pitch };

    return _OGCWebServiceHelper2.default.getColorTextureByUrl(urld, layer.networkOptions).then(function (texture) {
        result.texture = texture;
        result.texture.extent = extent;
        if (layer.transparent) {
            texture.premultiplyAlpha = true;
        }
        if (tileCoords) {
            result.texture.coords = tileCoords;
        } else {
            result.texture.coords = coords;
            // LayeredMaterial expects coords.zoom to exist, and describe the
            // precision of the texture (a la WMTS).
            result.texture.coords.zoom = targetLevel;
        }
        return result;
    });
}

function executeCommand(command) {
    var tile = command.requester;

    var layer = command.layer;
    var getTextureFunction = tile.extent.crs() == layer.projection ? getColorTexture : getColorTextures;

    return getTextureFunction(tile, layer, command.targetLevel);
}

// In the case where the tilematrixset of the tile don't correspond to the projection of the layer
// when the projection of the layer corresponds to a tilematrixset inside the tile, like the PM
function getColorTextures(tile, layer, targetLevel) {
    if (tile.material === null) {
        return _promise2.default.resolve();
    }
    var promises = [];
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(layer.getCoords(tile)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var coord = _step.value;

            promises.push(getColorTexture(tile, layer, targetLevel, coord));
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return _promise2.default.all(promises);
}

exports.default = {
    preprocessDataLayer: preprocessDataLayer,
    executeCommand: executeCommand,
    tileTextureCount: tileTextureCount,
    tileInsideLimit: tileInsideLimit
};