'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _OGCWebServiceHelper = require('./OGCWebServiceHelper');

var _OGCWebServiceHelper2 = _interopRequireDefault(_OGCWebServiceHelper);

var _URLBuilder = require('./URLBuilder');

var _URLBuilder2 = _interopRequireDefault(_URLBuilder);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Generated On: 2015-10-5
 * Class: WMTSProvider
 * Description: Fournisseur de données à travers un flux WMTS
 */

var coordTile = new _Extent2.default('WMTS:WGS84', 0, 0, 0);

var supportedFormats = new _map2.default([['image/png', getColorTextures], ['image/jpg', getColorTextures], ['image/jpeg', getColorTextures], ['image/x-bil;bits=32', getXbilTexture]]);

function preprocessDataLayer(layer) {
    layer.fx = layer.fx || 0.0;

    layer.format = layer.format || 'image/png';
    if (!supportedFormats.has(layer.format)) {
        throw new Error('Layer ' + layer.name + ': unsupported layer.format \'' + layer.format + '\', must be one of \'' + (0, _from2.default)(supportedFormats.keys()).join('\', \'') + '\'');
    }

    if (layer.protocol === 'wmts') {
        var options = layer.options;
        options.version = options.version || '1.0.0';
        options.tileMatrixSet = options.tileMatrixSet || 'WGS84';
        options.style = options.style || 'normal';
        options.projection = options.projection || 'EPSG:3857';
        var newBaseUrl = '' + layer.url + ('?LAYER=' + options.name) + ('&FORMAT=' + layer.format) + '&SERVICE=WMTS' + ('&VERSION=' + options.version) + '&REQUEST=GetTile' + ('&STYLE=' + options.style) + ('&TILEMATRIXSET=' + options.tileMatrixSet);

        newBaseUrl += '&TILEMATRIX=%TILEMATRIX&TILEROW=%ROW&TILECOL=%COL';

        if (!layer.options.zoom) {
            var arrayLimits = (0, _keys2.default)(options.tileMatrixSetLimits);
            var size = arrayLimits.length;
            var maxZoom = Number(arrayLimits[size - 1]);


            layer.options.zoom = {
                min: maxZoom - size + 1,
                max: maxZoom
            };
        }
        layer.url = newBaseUrl;
    }
    layer.getCoords = function (node) {
        _OGCWebServiceHelper2.default.computeTileMatrixSetCoordinates(node, this.options.tileMatrixSet);
        return node.wmtsCoords[this.options.tileMatrixSet];
    };
    layer.getZoom = function (node) {
        return layer.getCoords(node)[0].zoom;
    };
    layer.options.zoom = layer.options.zoom || { min: 2, max: 20 };
}

/**
 * return texture float alpha THREE.js of MNT
 * @param {TileMesh} tile
 * @param {Layer} layer
 * @param {number} targetZoom
 * @returns {Promise<portableXBIL>}
 */
function getXbilTexture(tile, layer, targetZoom) {
    var pitch = new THREE.Vector4(0.0, 0.0, 1.0, 1.0);
    var coordWMTS = layer.getCoords(tile)[0];

    if (targetZoom && targetZoom !== coordWMTS.zoom) {
        coordWMTS = _OGCWebServiceHelper2.default.WMTS_WGS84Parent(coordWMTS, targetZoom, pitch);
    }

    var urld = _URLBuilder2.default.xyz(coordWMTS, layer);

    return _OGCWebServiceHelper2.default.getXBilTextureByUrl(urld, layer.networkOptions).then(function (texture) {
        texture.coords = coordWMTS;
        return {
            texture: texture,
            pitch: pitch,
            min: !texture.min ? 0 : texture.min,
            max: !texture.max ? 0 : texture.max
        };
    });
}

/**
 * Return texture RGBA THREE.js of orthophoto
 * TODO : RGBA --> RGB remove alpha canal
 * @param {{zoom:number,row:number,col:number}} coordWMTS
 * @param {Layer} layer
 * @param {number} targetZoom
 * @returns {Promise<Texture>}
 */
function getColorTexture(coordWMTS, layer, targetZoom) {
    var pitch = new THREE.Vector4(0.0, 0.0, 1.0, 1.0);
    if (targetZoom && targetZoom !== coordWMTS.zoom) {
        coordWMTS = _OGCWebServiceHelper2.default.WMTS_WGS84Parent(coordWMTS, targetZoom, pitch);
    }

    var urld = _URLBuilder2.default.xyz(coordWMTS, layer);
    return _OGCWebServiceHelper2.default.getColorTextureByUrl(urld, layer.networkOptions).then(function (texture) {
        var result = {};
        result.texture = texture;
        result.texture.coords = coordWMTS;
        result.pitch = pitch;
        if (layer.transparent) {
            texture.premultiplyAlpha = true;
        }

        return result;
    });
}

function executeCommand(command) {
    var layer = command.layer;
    var tile = command.requester;
    return supportedFormats.get(layer.format)(tile, layer, command.targetLevel);
}

function tileTextureCount(tile, layer) {
    var tileMatrixSet = layer.options.tileMatrixSet;
    _OGCWebServiceHelper2.default.computeTileMatrixSetCoordinates(tile, tileMatrixSet);
    return layer.getCoords(tile).length;
}

function tileInsideLimit(tile, layer, targetLevel) {
    // This layer provides data starting at level = layer.options.zoom.min
    // (the zoom.max property is used when building the url to make
    //  sure we don't use invalid levels)
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(layer.getCoords(tile)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var coord = _step.value;

            var c = coord;
            // override
            if (targetLevel < c.zoom) {
                _OGCWebServiceHelper2.default.WMTS_WGS84Parent(coord, targetLevel, undefined, coordTile);
                c = coordTile;
            }
            if (c.zoom < layer.options.zoom.min || c.zoom > layer.options.zoom.max) {
                return false;
            }
            if (layer.options.tileMatrixSetLimits) {
                if (c.row < layer.options.tileMatrixSetLimits[c.zoom].minTileRow || c.row > layer.options.tileMatrixSetLimits[c.zoom].maxTileRow || c.col < layer.options.tileMatrixSetLimits[c.zoom].minTileCol || c.col > layer.options.tileMatrixSetLimits[c.zoom].maxTileCol) {
                    return false;
                }
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return true;
}

function getColorTextures(tile, layer, targetZoom) {
    if (tile.material === null) {
        return _promise2.default.resolve();
    }
    var promises = [];
    var bcoord = layer.getCoords(tile);

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = (0, _getIterator3.default)(bcoord), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var coordWMTS = _step2.value;

            promises.push(getColorTexture(coordWMTS, layer, targetZoom));
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return _promise2.default.all(promises);
}

exports.default = {
    preprocessDataLayer: preprocessDataLayer,
    executeCommand: executeCommand,
    tileTextureCount: tileTextureCount,
    tileInsideLimit: tileInsideLimit
};