'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var pt = new THREE.Vector2();

function _moveTo(ctx, x, y, scale, origin) {
    pt.x = x - origin.x;
    pt.y = y - origin.y;
    pt.multiply(scale);
    ctx.moveTo(Math.round(pt.x), Math.round(pt.y));
}

function _lineTo(ctx, x, y, scale, origin) {
    pt.x = x - origin.x;

    pt.y = y - origin.y;
    pt.multiply(scale);
    ctx.lineTo(Math.round(pt.x), Math.round(pt.y));
}

function drawPolygon(ctx, vertices) {
    var indices = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [{ offset: 0, count: 1 }];
    var origin = arguments[3];
    var scale = arguments[4];
    var properties = arguments[5];
    var style = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {};
    var size = arguments[7];

    if (vertices.length === 0) {
        return;
    }

    if (style.length) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = (0, _getIterator3.default)(style), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var s = _step.value;

                _drawPolygon(ctx, vertices, indices, origin, scale, properties, s, size);
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }
    } else {
        _drawPolygon(ctx, vertices, indices, origin, scale, properties, style, size);
    }
}

function _drawPolygon(ctx, vertices, indices, origin, scale) {
    var properties = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};
    var style = arguments[6];
    var size = arguments[7];

    // build contour
    ctx.beginPath();
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = (0, _getIterator3.default)(indices), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var indice = _step2.value;

            var offset = indice.offset * size;
            var count = offset + indice.count * size;
            _moveTo(ctx, vertices[offset], vertices[offset + 1], scale, origin);
            for (var j = offset + size; j < count; j += size) {
                _lineTo(ctx, vertices[j], vertices[j + 1], scale, origin);
            }
        }

        // draw line polygon
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    if (style.stroke || properties.stroke) {
        ctx.strokeStyle = style.stroke || properties.stroke;
        ctx.lineWidth = style.strokeWidth || properties['stroke-width'] || 2.0;
        ctx.globalAlpha = style.strokeOpacity || properties['stroke-opacity'] || 1.0;
        ctx.stroke();
    }

    // fill polygon
    if (indices && (style.fill || properties.fill)) {
        ctx.fillStyle = style.fill || properties.fill;
        ctx.globalAlpha = style.fillOpacity || properties['fill-opacity'] || 1.0;
        ctx.fill();
    }
}

function drawPoint(ctx, x, y, origin, scale) {
    var style = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};

    pt.x = x - origin.x;
    pt.y = y - origin.y;
    pt.multiply(scale);

    ctx.beginPath();
    ctx.arc(Math.round(pt.x), Math.round(pt.y), Math.round(style.radius) || 3, 0, 2 * Math.PI, false);
    ctx.fillStyle = style.fill || 'white';
    ctx.fill();
    ctx.lineWidth = style.lineWidth || 1.0;
    ctx.strokeStyle = style.stroke || 'red';
    ctx.stroke();
}

function drawFeature(ctx, feature, origin, scale, extent) {
    var style = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};

    var gStyle = style;
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
        for (var _iterator3 = (0, _getIterator3.default)(feature.geometry), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var geometry = _step3.value;

            var properties = geometry.properties;

            if (typeof style == 'function') {
                gStyle = style(properties, feature);
            }
            if (feature.type === 'point') {
                drawPoint(ctx, feature.vertices[0], feature.vertices[1], origin, scale, gStyle);
            } else if (feature.type === 'multipoint') {
                for (var i = 0; i < feature.vertices.length; i += feature.size) {
                    drawPoint(ctx, feature.vertices[i], feature.vertices[i + 1], origin, scale, gStyle);
                }
            } else if (geometry.extent.intersectsExtent(extent)) {
                drawPolygon(ctx, feature.vertices, geometry.indices, origin, scale, properties, gStyle, feature.size);
            }
        }
    } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
            }
        } finally {
            if (_didIteratorError3) {
                throw _iteratorError3;
            }
        }
    }
}

exports.default = {
    // backgroundColor is a THREE.Color to specify a color to fill the texture
    // with, given there is no feature passed in parameter
    createTextureFromFeature: function createTextureFromFeature(collection, extent, sizeTexture, style, backgroundColor) {
        var texture = void 0;

        if (collection) {
            // A texture is instancied drawn canvas
            // origin and dimension are used to transform the feature's coordinates to canvas's space
            var origin = new THREE.Vector2(extent.west(), extent.south());
            var dimension = extent.dimensions();
            var c = document.createElement('canvas');

            c.width = sizeTexture;
            c.height = sizeTexture;
            var ctx = c.getContext('2d');
            if (backgroundColor) {
                ctx.fillStyle = backgroundColor.getStyle();
                ctx.fillRect(0, 0, sizeTexture, sizeTexture);
            }
            ctx.globalCompositeOperation = style.globalCompositeOperation || 'source-over';

            var scale = new THREE.Vector2(ctx.canvas.width / dimension.x, ctx.canvas.width / dimension.y);

            // Draw the canvas
            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = (0, _getIterator3.default)(collection.features), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    var feature = _step4.value;

                    var ex = feature.crs == extent.crs() ? extent : extent.as(feature.crs);
                    drawFeature(ctx, feature, origin, scale, ex, style);
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }

            texture = new THREE.CanvasTexture(c);
            texture.flipY = false;
        } else if (backgroundColor) {
            var data = new Uint8Array(3);
            data[0] = backgroundColor.r * 255;
            data[1] = backgroundColor.g * 255;
            data[2] = backgroundColor.b * 255;
            texture = new THREE.DataTexture(data, 1, 1, THREE.RGBFormat);
            texture.needsUpdate = true;
        } else {
            texture = new THREE.Texture();
        }

        return texture;
    }
};