'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _sign = require('babel-runtime/core-js/math/sign');

var _sign2 = _interopRequireDefault(_sign);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _MainLoop = require('../../Core/MainLoop');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Note: we could use existing three.js controls (like https://github.com/mrdoob/three.js/blob/dev/examples/js/controls/FirstPersonControls.js)
// but including these controls in itowns allows use to integrate them tightly with itowns.
// Especially the existing controls are expecting a continuous update loop while we have a pausable one (so our controls use .notifyChange when needed)

function limitRotation(camera3D, rot, verticalFOV) {
    // Limit vertical rotation (look up/down) to make sure the user cannot see
    // outside of the cone defined by verticalFOV
    var limit = THREE.Math.degToRad(verticalFOV - camera3D.fov) * 0.5;
    return THREE.Math.clamp(rot, -limit, limit);
}

function applyRotation(view, camera3D, state) {
    camera3D.quaternion.setFromUnitVectors(new THREE.Vector3(0, 1, 0), camera3D.up);

    camera3D.rotateY(state.rotateY);
    camera3D.rotateX(state.rotateX);

    view.notifyChange(view.camera.camera3D);
}

var MOVEMENTS = {
    38: { method: 'translateZ', sign: -1 }, // FORWARD: up key
    40: { method: 'translateZ', sign: 1 }, // BACKWARD: down key
    37: { method: 'translateX', sign: -1 }, // STRAFE_LEFT: left key
    39: { method: 'translateX', sign: 1 }, // STRAFE_RIGHT: right key
    33: { method: 'translateY', sign: 1 }, // UP: PageUp key
    34: { method: 'translateY', sign: -1 } // DOWN: PageDown key
};

var FirstPersonControls = function (_THREE$EventDispatche) {
    (0, _inherits3.default)(FirstPersonControls, _THREE$EventDispatche);

    /**
     * @Constructor
     * @param {View} view
     * @param {object} options
     * @param {boolean} options.focusOnClick - whether or not to focus the renderer domElement on click
     * @param {boolean} options.focusOnMouseOver - whether or not to focus when the mouse is over the domElement
     * @param {boolean} options.moveSpeed - if > 0, pressing the arrow keys will move the camera
     * @param {number} options.verticalFOV - define the max visible vertical angle of the scene in degrees (default 180)
     * @param {number} options.panoramaRatio - alternative way to specify the max vertical angle when using a panorama.
     * You can specify the panorama width/height ratio and the verticalFOV will be computed automatically
     * @param {boolean} options.disableEventListeners - if true, the controls will not self listen to mouse/key events.
     * You'll have to manually forward the events to the appropriate functions: onMouseDown, onMouseMove, onMouseUp,
     * onKeyUp, onKeyDown and onMouseWheel.
     */
    function FirstPersonControls(view) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        (0, _classCallCheck3.default)(this, FirstPersonControls);

        var _this = (0, _possibleConstructorReturn3.default)(this, (FirstPersonControls.__proto__ || (0, _getPrototypeOf2.default)(FirstPersonControls)).call(this));

        _this.camera = view.camera.camera3D;
        _this.view = view;
        _this.moves = new _set2.default();
        if (options.panoramaRatio) {
            var radius = options.panoramaRatio * 200 / (2 * Math.PI);
            options.verticalFOV = options.panoramaRatio == 2 ? 180 : THREE.Math.radToDeg(2 * Math.atan(200 / (2 * radius)));
        }
        options.verticalFOV = options.verticalFOV || 180;
        options.moveSpeed = options.moveSpeed === undefined ? 10 : options.moveSpeed; // backward or forward move speed in m/s
        _this.options = options;

        _this._isMouseDown = false;
        _this._onMouseDownMouseX = 0;
        _this._onMouseDownMouseY = 0;

        _this._state = {
            rotateX: 0,
            rotateY: 0,
            snapshot: function snapshot() {
                return {
                    rotateX: this.rotateX,
                    rotateY: this.rotateY
                };
            }
        };
        _this.reset();

        var domElement = view.mainLoop.gfxEngine.renderer.domElement;
        if (!options.disableEventListeners) {
            domElement.addEventListener('mousedown', _this.onMouseDown.bind(_this), false);
            domElement.addEventListener('touchstart', _this.onMouseDown.bind(_this), false);
            domElement.addEventListener('mousemove', _this.onMouseMove.bind(_this), false);
            domElement.addEventListener('touchmove', _this.onMouseMove.bind(_this), false);
            domElement.addEventListener('mouseup', _this.onMouseUp.bind(_this), false);
            domElement.addEventListener('touchend', _this.onMouseUp.bind(_this), false);
            domElement.addEventListener('keyup', _this.onKeyUp.bind(_this), true);
            domElement.addEventListener('keydown', _this.onKeyDown.bind(_this), true);
            domElement.addEventListener('mousewheel', _this.onMouseWheel.bind(_this), false);
            domElement.addEventListener('DOMMouseScroll', _this.onMouseWheel.bind(_this), false); // firefox
        }

        _this.view.addFrameRequester(_MainLoop.MAIN_LOOP_EVENTS.AFTER_CAMERA_UPDATE, _this.update.bind(_this));

        // focus policy
        if (options.focusOnMouseOver) {
            domElement.addEventListener('mouseover', function () {
                return domElement.focus();
            });
        }
        if (options.focusOnClick) {
            domElement.addEventListener('click', function () {
                return domElement.focus();
            });
        }
        return _this;
    }

    (0, _createClass3.default)(FirstPersonControls, [{
        key: 'isUserInteracting',
        value: function isUserInteracting() {
            return this.moves.size !== 0 && !this._isMouseDown;
        }

        /**
         * Resets the controls internal state to match the camera' state.
         * This must be called when manually modifying the camera's position or rotation.
         * @param {boolean} preserveRotationOnX - if true, the look up/down rotation will
         * not be copied from the camera
         */

    }, {
        key: 'reset',
        value: function reset() {
            var preserveRotationOnX = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

            // Compute the correct init state, given the calculus in applyRotation:
            // cam.quaternion = q * r
            // => r = inverse(q) * cam.quaterion
            // q is the quaternion derived from the up vector
            var q = new THREE.Quaternion().setFromUnitVectors(new THREE.Vector3(0, 1, 0), this.camera.up);
            q.inverse();
            // compute r
            var r = this.camera.quaternion.clone().premultiply(q);
            // tranform it to euler
            var e = new THREE.Euler(0, 0, 0, 'YXZ').setFromQuaternion(r);

            if (!preserveRotationOnX) {
                this._state.rotateX = e.x;
            }
            this._state.rotateY = e.y;
        }

        /**
         * Updates the camera position / rotation based on occured input events.
         * This is done automatically when needed but can also be done if needed.
         * @param {number} dt - ellpased time since last update in seconds
         * @param {boolean} updateLoopRestarted - true if itowns' update loop just restarted
         * @param {boolean} force - set to true if you want to force the update, even if it
         * appears unneeded.
         */

    }, {
        key: 'update',
        value: function update(dt, updateLoopRestarted, force) {
            // dt will not be relevant when we just started rendering, we consider a 1-frame move in this case
            if (updateLoopRestarted) {
                dt = 16;
            }

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)(this.moves), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var move = _step.value;

                    if (move.method === 'translateY') {
                        this.camera.position.z += move.sign * this.options.moveSpeed * dt / 1000;
                    } else {
                        this.camera[move.method](move.sign * this.options.moveSpeed * dt / 1000);
                    }
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            if (this._isMouseDown === true || force === true) {
                applyRotation(this.view, this.camera, this._state);
            }

            if (this.moves.size) {
                this.view.notifyChange(this.view.camera.camera3D);
            }
        }

        // Event callback functions
        // Mouse movement handling

    }, {
        key: 'onMouseDown',
        value: function onMouseDown(event) {
            event.preventDefault();
            this._isMouseDown = true;

            var coords = this.view.eventToViewCoords(event);
            this._onMouseDownMouseX = coords.x;
            this._onMouseDownMouseY = coords.y;

            this._stateOnMouseDown = this._state.snapshot();
        }
    }, {
        key: 'onMouseUp',
        value: function onMouseUp() {
            this._isMouseDown = false;
        }
    }, {
        key: 'onMouseMove',
        value: function onMouseMove(event) {
            if (this._isMouseDown === true) {
                // in rigor we have tan(theta) = tan(cameraFOV) * deltaH / H
                // (where deltaH is the vertical amount we moved, and H the renderer height)
                // we loosely approximate tan(x) by x
                var pxToAngleRatio = THREE.Math.degToRad(this.camera.fov) / this.view.mainLoop.gfxEngine.height;

                var coords = this.view.eventToViewCoords(event);

                // update state based on pointer movement
                this._state.rotateY = (coords.x - this._onMouseDownMouseX) * pxToAngleRatio + this._stateOnMouseDown.rotateY;
                this._state.rotateX = limitRotation(this.camera, (coords.y - this._onMouseDownMouseY) * pxToAngleRatio + this._stateOnMouseDown.rotateX, this.options.verticalFOV);

                applyRotation(this.view, this.camera, this._state);
            }
        }

        // Mouse wheel

    }, {
        key: 'onMouseWheel',
        value: function onMouseWheel(event) {
            var delta = 0;
            if (event.wheelDelta !== undefined) {
                delta = -event.wheelDelta;
                // Firefox
            } else if (event.detail !== undefined) {
                delta = event.detail;
            }

            this.camera.fov = THREE.Math.clamp(this.camera.fov + (0, _sign2.default)(delta), 10, Math.min(100, this.options.verticalFOV));

            this.camera.updateProjectionMatrix();

            this._state.rotateX = limitRotation(this.camera, this._state.rotateX, this.options.verticalFOV);

            applyRotation(this.view, this.camera, this._state);
        }

        // Keyboard handling

    }, {
        key: 'onKeyUp',
        value: function onKeyUp(e) {
            var move = MOVEMENTS[e.keyCode];
            if (move) {
                this.moves.delete(move);
                this.view.notifyChange(undefined, false);
                e.preventDefault();
            }
        }
    }, {
        key: 'onKeyDown',
        value: function onKeyDown(e) {
            var move = MOVEMENTS[e.keyCode];
            if (move) {
                this.moves.add(move);
                this.view.notifyChange(undefined, false);
                e.preventDefault();
            }
        }
    }]);
    return FirstPersonControls;
}(THREE.EventDispatcher);

exports.default = FirstPersonControls;