'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CONTROL_KEYS = {
    LEFT: 37,
    UP: 38,
    RIGHT: 39,
    BOTTOM: 40,
    SPACE: 32,
    SHIFT: 16,
    CTRL: 17,
    S: 83
};

/**
 * @typedef {Object} state
 * @property {boolean} enable=true indicate whether the state is enable or not
 * @property {Number} [mouseButton] the mouse button to enter this state
 * @property {Number} [keyboard] the keyboard to enter this state
 * @property {Number} [finger] the number of fingers on the pad to enter this state
 */

/**
 * It represents the control's states.
 * Each {@link state} is a control mode of the camera and how to interact with
 * the interface to activate this mode.
 * @class StateControl
 *
 */

var StateControl = function () {
    function StateControl() {
        (0, _classCallCheck3.default)(this, StateControl);

        this.NONE = {};
        /**
         * The camera loot at target and moves at a constant distance from it
         */
        this.ORBIT = {
            mouseButton: THREE.MOUSE.LEFT,
            keyboard: CONTROL_KEYS.CTRL,
            enable: true,
            finger: 2
        };
        /**
         * The camera loot at target and moves forward and backward from this point
         */
        this.DOLLY = {
            mouseButton: THREE.MOUSE.MIDDLE,
            enable: true
        };
        /**
         * the camera moves parallel to the current view plane
         */
        this.PAN = {
            mouseButton: THREE.MOUSE.RIGHT,
            up: CONTROL_KEYS.UP,
            bottom: CONTROL_KEYS.BOTTOM,
            left: CONTROL_KEYS.LEFT,
            right: CONTROL_KEYS.RIGHT,
            enable: true,
            finger: 3
        };
        this.MOVE_GLOBE = {
            mouseButton: THREE.MOUSE.LEFT,
            enable: true,
            finger: 1
        };
        /**
         * the camera and target camera rotate around the globe
         */
        this.PANORAMIC = {
            mouseButton: THREE.MOUSE.LEFT,
            keyboard: CONTROL_KEYS.SHIFT,
            enable: true
        };
        /**
         * object selection
         */
        this.SELECT = {
            mouseButton: THREE.MOUSE.LEFT,
            keyboard: CONTROL_KEYS.S,
            enable: true
        };
    }

    /**
     * get the state corresponding to the mouse button and the keyboard key
     * @param      {Number}  mouseButton  The mouse button
     * @param      {Number}  keyboard     The keyboard
     * @return     {state}  the state corresponding
     */


    (0, _createClass3.default)(StateControl, [{
        key: 'inputToState',
        value: function inputToState(mouseButton, keyboard) {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)((0, _keys2.default)(this)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var key = _step.value;

                    var state = this[key];
                    if (state.enable && state.mouseButton === mouseButton && state.keyboard === keyboard) {
                        return state;
                    }
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            return this.NONE;
        }

        /**
         * get the state corresponding to the number of finger on the pad
         *
         * @param      {Number}  finger  The number of finger
         * @return     {state}  the state corresponding
         */

    }, {
        key: 'touchToState',
        value: function touchToState(finger) {
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = (0, _getIterator3.default)((0, _keys2.default)(this)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var key = _step2.value;

                    var state = this[key];
                    if (state.enable && finger == state.finger) {
                        return state;
                    }
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }

            return this.NONE;
        }
    }]);
    return StateControl;
}();

exports.default = StateControl;