'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _togeojson = require('togeojson');

var _togeojson2 = _interopRequireDefault(_togeojson);

var _Source2 = require('./Source');

var _Source3 = _interopRequireDefault(_Source2);

var _Fetcher = require('../Provider/Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _GeoJsonParser = require('../Parser/GeoJsonParser');

var _GeoJsonParser2 = _interopRequireDefault(_GeoJsonParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getExtentFromGpxFile(file) {
    var bound = file.getElementsByTagName('bounds')[0];
    if (bound) {
        var west = bound.getAttribute('minlon');
        var east = bound.getAttribute('maxlon');
        var south = bound.getAttribute('minlat');
        var north = bound.getAttribute('maxlat');
        return new _Extent2.default('EPSG:4326', west, east, south, north);
    }
    return new _Extent2.default('EPSG:4326', -180, 180, -90, 90);
}

// TODO move and refacto
function fileParser(text) {
    var parsedFile = void 0;
    var trimmedText = text.trim();
    // We test the start of the string to choose a parser
    if (trimmedText.startsWith('<')) {
        // if it's an xml file, then it can be kml or gpx
        var parser = new DOMParser();
        var file = parser.parseFromString(text, 'application/xml');
        if (file.documentElement.tagName.toLowerCase() === 'kml') {
            parsedFile = _togeojson2.default.kml(file);
        } else if (file.documentElement.tagName.toLowerCase() === 'gpx') {
            parsedFile = _togeojson2.default.gpx(file);
            var line = parsedFile.features.find(function (e) {
                return e.geometry.type == 'LineString';
            });
            line.properties.stroke = 'red';
            parsedFile.extent = getExtentFromGpxFile(file);
        } else if (file.documentElement.tagName.toLowerCase() === 'parsererror') {
            throw new Error('Error parsing XML document');
        } else {
            throw new Error('Unsupported xml file, only valid KML and GPX are supported, but no <gpx> or <kml> tag found.', file);
        }
    } else if (trimmedText.startsWith('{') || trimmedText.startsWith('[')) {
        parsedFile = JSON.parse(text);
        if (parsedFile.type !== 'Feature' && parsedFile.type !== 'FeatureCollection') {
            throw new Error('This json is not a GeoJSON');
        }
    } else {
        throw new Error('Unsupported file: only well-formed KML, GPX or GeoJSON are supported');
    }

    return parsedFile;
}

var FileSource = function (_Source) {
    (0, _inherits3.default)(FileSource, _Source);

    /**
     * File source to use file in {@link GeometryLayer}, {@link ColorLayer} or {@link ElevationLayer}.
     * @constructor
     * @extends Source
     *
     * @param      {sourceParams}  source  The source
     * @param      {string}  source.projection  Data system projection, it needed to parse data
     * @param      {string}  crsOut  crd output data
     *
     * @example <caption>add geometry layer with geojson file </caption>
     * const ariege = new itowns.GeometryLayer('ariege', new itowns.THREE.Group());
     * ariege.update = itowns.FeatureProcessing.update;
     * ariege.convert = itowns.Feature2Mesh.convert({
     *      color: () => new itowns.THREE.Color(0xffcc00),
     *      extrude: () => 5000,
     * });
     * ariege.source = {
     *      protocol: 'file',
     *      url: 'https://raw.githubusercontent.com/gregoiredavid/france-geojson/master/departements/09-ariege/departement-09-ariege.geojson',
     *      projection: 'EPSG:4326',
     *      format: 'application/json',
     *      zoom: { min: 7, max: 7 },
     * };
     * view.addLayer(ariege);
     *
     * @example <caption>add color layer with geojson file </caption>
     * globeView.addLayer({
     *     type: 'color',
     *     id: 'Gpx',
     *     name: 'Ultra 2009',
     *     transparent: true,
     *     source: {
     *         url: 'https://raw.githubusercontent.com/iTowns/iTowns2-sample-data/master/ULTRA2009.gpx',
     *         protocol: 'file',
     *         projection: 'EPSG:4326',
     *     },
     * });
     *
     */
    function FileSource(source, crsOut) {
        (0, _classCallCheck3.default)(this, FileSource);

        if (!source.projection) {
            throw new Error('source.projection is required in FileSource');
        }

        var _this = (0, _possibleConstructorReturn3.default)(this, (FileSource.__proto__ || (0, _getPrototypeOf2.default)(FileSource)).call(this, source));

        _this.url = source.url;
        _this.parsedData = [];
        _this.zoom = source.zoom || { min: 5, max: 21 };
        var options = {
            buildExtent: source.toTexture,
            crsIn: _this.projection,
            crsOut: crsOut,
            withNormal: !source.toTexture,
            withAltitude: !source.toTexture,
            mergeFeatures: true
        };

        _this.whenReady = _Fetcher2.default.text(_this.url, source.networkOptions).then(fileParser).then(function (parsedFile) {
            return _GeoJsonParser2.default.parse(parsedFile, options).then(function (feature) {
                feature.style = parsedFile.style;
                _this.parsedData = feature;
            });
        });
        return _this;
    }

    (0, _createClass3.default)(FileSource, [{
        key: 'urlFromExtent',
        value: function urlFromExtent(extent) {
            return this.url + ',' + extent.crs();
        }
    }, {
        key: 'extentInsideLimit',
        value: function extentInsideLimit(extent) {
            var dataExtent = this.parsedData.extent;
            var localExtent = this.projection == extent.crs() ? extent : extent.as(this.projection);
            return (extent.zoom == undefined || !(extent.zoom < this.zoom.min || extent.zoom > this.zoom.max)) && dataExtent.intersectsExtent(localExtent);
        }
    }]);
    return FileSource;
}(_Source3.default);

exports.default = FileSource;