'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _flatbush = require('flatbush');

var _flatbush2 = _interopRequireDefault(_flatbush);

var _Source2 = require('./Source');

var _Source3 = _interopRequireDefault(_Source2);

var _Fetcher = require('../Provider/Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _selectImagesFromSpatialIndex(index, images, extent) {
    return index.search(extent.west(), extent.south(), extent.east(), extent.north()).map(function (i) {
        return images[i];
    });
}

function buildUrl(layer, image) {
    return layer.url.href.substr(0, layer.url.href.lastIndexOf('/') + 1) + image;
}

var StaticSource = function (_Source) {
    (0, _inherits3.default)(StaticSource, _Source);

    /**
     * Images source to panorama layer {@link PanoramaLayer}
     *
     * @constructor
     * @extends Source
     * @param {sourceParams}  source  The source
     * @param {string}  source.extent It's extent of panoramic's images
    */
    function StaticSource(source) {
        (0, _classCallCheck3.default)(this, StaticSource);

        if (!source.extent) {
            throw new Error('layer.extent is required');
        }

        var _this = (0, _possibleConstructorReturn3.default)(this, (StaticSource.__proto__ || (0, _getPrototypeOf2.default)(StaticSource)).call(this, source));

        _this.zoom = { min: 0, max: 0 };
        _this.url = new URL(source.url, window.location);
        _this.whenReady = _Fetcher2.default.json(_this.url.href).then(function (metadata) {
            _this.images = [];
            // eslint-disable-next-line guard-for-in
            for (var image in metadata) {
                var extent = new (Function.prototype.bind.apply(_Extent2.default, [null].concat([_this.projection], (0, _toConsumableArray3.default)(metadata[image]))))();
                _this.images.push({
                    image: image,
                    extent: extent
                });
            }
            if (!_this.images.length) {
                return;
            }
            _this._spatialIndex = (0, _flatbush2.default)(_this.images.length);
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)(_this.images), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var _image = _step.value;

                    _this._spatialIndex.add(_image.extent.west(), _image.extent.south(), _image.extent.east(), _image.extent.north());
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            _this._spatialIndex.finish();
        }).then(function () {
            if (!_this.format) {
                // fetch the first image to detect format
                if (_this.images.length) {
                    var url = buildUrl(_this, _this.images[0].image);
                    return fetch(url, _this.networkOptions).then(function (response) {
                        _this.format = response.headers.get('Content-type');
                        if (_this.format === 'application/octet-stream') {
                            _this.format = 'image/x-bil';
                        }
                        if (!_this.format) {
                            throw new Error(_this.name + ': could not detect layer format, please configure \'layer.format\'.');
                        }
                    });
                }
            }
        });
        return _this;
    }

    (0, _createClass3.default)(StaticSource, [{
        key: 'urlFromExtent',
        value: function urlFromExtent(extent) {
            var selection = this.getSourceExtents(extent);
            return this.url.href.substr(0, this.url.href.lastIndexOf('/') + 1) + selection.image;
        }
    }, {
        key: 'handlingError',
        value: function handlingError(err, url) {
            console.error('Source static: ' + this.url + ', ' + err.response.status + ' error while trying to fetch data. Url was ' + url + '.', err);
        }
    }, {
        key: 'canTileTextureBeImproved',
        value: function canTileTextureBeImproved(extent, texture) {
            if (!this.images) {
                return false;
            }
            var s = this.getSourceExtents(extent);

            if (!s) {
                return false;
            }
            if (!texture.image) {
                return true;
            }

            var urlTexture = texture.image.currentSrc;
            var urlSource = this.url.href.substr(0, this.url.href.lastIndexOf('/') + 1) + s.image;

            return urlSource != urlTexture;
        }
    }, {
        key: 'getSourceExtents',
        value: function getSourceExtents(extent) {
            // select the smallest image entirely covering the tile
            extent = extent.crs() === this.extent.crs() ? extent : extent.as(this.extent.crs());
            var candidates = _selectImagesFromSpatialIndex(this._spatialIndex, this.images, extent);

            var selection = void 0;
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = (0, _getIterator3.default)(candidates), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var entry = _step2.value;

                    if (extent.isInside(entry.extent)) {
                        if (!selection) {
                            selection = entry;
                        } else {
                            var d = selection.extent.dimensions();
                            var e = entry.extent.dimensions();
                            if (e.x <= d.x && e.y <= d.y) {
                                selection = entry;
                            }
                        }
                    }
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }

            return selection;
        }
    }, {
        key: 'extentInsideLimit',
        value: function extentInsideLimit(extent) {
            return extent.intersectsExtent(this.extent);
        }
    }]);
    return StaticSource;
}(_Source3.default);

exports.default = StaticSource;