'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _Source2 = require('./Source');

var _Source3 = _interopRequireDefault(_Source2);

var _URLBuilder = require('../Provider/URLBuilder');

var _URLBuilder2 = _interopRequireDefault(_URLBuilder);

var _Extent = require('../Core/Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TMSSource = function (_Source) {
    (0, _inherits3.default)(TMSSource, _Source);

    /**
     * Tiled images source
     * @constructor
     * @extends Source
     * @param {sourceParams}  source
     * @param {string} [source.origin] origin row coordinate: 'top' or 'bottom'
     * @param {Object} [source.zoom]
     * @param {number} [source.zoom.min] layer's zoom minimum
     * @param {number} [source.zoom.max] layer's zoom maximum
     * @param {string} [source.tileMatrixSet='WGS84']  define tile matrix set of tms layer (ex: 'PM', 'WGS84')
     *
     * @example <caption>Add color layer with tms source</caption>
     * const colorlayer = new ColorLayer('OPENSM', {
     *     source: {
     *         protocol: 'xyz',
     *         format: 'image/png',
     *         url: 'http://osm.io/styles/${z}/${x}/${y}.png',
     *         attribution: {
     *             name: 'OpenStreetMap',
     *             url: 'http://www.openstreetmap.org/',
     *         },
     *         tileMatrixSet: 'PM',
     *     }
     * });
     * // Add the layer
     * view.addLayer(colorlayer);
    */
    function TMSSource(source) {
        (0, _classCallCheck3.default)(this, TMSSource);

        var _this = (0, _possibleConstructorReturn3.default)(this, (TMSSource.__proto__ || (0, _getPrototypeOf2.default)(TMSSource)).call(this, source));

        if (!source.extent) {
            // default to the full 3857 extent
            _this.extent = new _Extent2.default('EPSG:3857', -20037508.342789244, 20037508.342789244, -20037508.342789255, 20037508.342789244);
        }

        _this.zoom = source.zoom || { min: 0, max: 18 };

        _this.origin = source.origin || (source.protocol == 'xyz' ? 'top' : 'bottom');

        _this.format = _this.format || 'image/png';
        _this.url = source.url;
        if (source.tileMatrixSet) {
            _this.tileMatrixSet = source.tileMatrixSet;
        } else if (_this.projection == 'EPSG:3857') {
            _this.tileMatrixSet = 'PM';
        } else {
            _this.tileMatrixSet = 'WGS84';
        }
        return _this;
    }

    (0, _createClass3.default)(TMSSource, [{
        key: 'urlFromExtent',
        value: function urlFromExtent(extent) {
            return _URLBuilder2.default.xyz(extent, this);
        }
    }, {
        key: 'handlingError',
        value: function handlingError(err) {
            console.warn('err ' + this.url, err);
        }
    }, {
        key: 'extentInsideLimit',
        value: function extentInsideLimit(extent) {
            // This layer provides data starting at level = layer.source.zoom.min
            // (the zoom.max property is used when building the url to make
            //  sure we don't use invalid levels)
            // TODO: add extent limit
            return extent.zoom >= this.zoom.min && extent.zoom <= this.zoom.max;
        }
    }]);
    return TMSSource;
}(_Source3.default);

exports.default = TMSSource;