'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _Source2 = require('./Source');

var _Source3 = _interopRequireDefault(_Source2);

var _URLBuilder = require('../Provider/URLBuilder');

var _URLBuilder2 = _interopRequireDefault(_URLBuilder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var WMSSource = function (_Source) {
    (0, _inherits3.default)(WMSSource, _Source);

    /**
     * Images source
     * @constructor
     * @extends Source
     * @param {sourceParams}  source
     * @param {string} source.name name of layer wms
     * @param {Extent} source.extent extent of wms source
     * @param {string} [source.style=''] style of layer wms
     * @param {number} [source.heightMapWidth=256] size texture in pixel
     * @param {string} [source.version='1.3.0'] wms version
     * @param {string} [source.axisOrder] wms axis order ('wsen' or 'swne')
     * @param {boolean} [source.transparent=false] source return texture with transparence
     * @param {Object} [source.zoom]
     * @param {number} [source.zoom.min] layer's zoom minimum
     * @param {number} [source.zoom.max] layer's zoom maximum
     *
     * @example <caption>Add color layer with wms source</caption>
     * const colorlayer = new ColorLayer('Region', {
     *     source: {
     *         url: 'https://wxs.fr/wms',
     *         protocol: 'wms',
     *         version: '1.3.0',
     *         name: 'REGION.2016',
     *         style: '',
     *         projection: 'EPSG:3857',
     *         extent: {
     *             west: '-6880639.13557728',
     *             east: '6215707.87974825',
     *             south: '-2438399.00148845',
     *             north: '7637050.03850605',
     *         },
     *         transparent: true,
     *     },
     * });
     * // Add the layer
     * view.addLayer(colorlayer);
     *
    */
    function WMSSource(source) {
        (0, _classCallCheck3.default)(this, WMSSource);

        if (!source.name) {
            throw new Error('source.name is required.');
        }

        if (!source.extent) {
            throw new Error('source.extent is required');
        }

        if (!source.projection) {
            throw new Error('source.projection is required');
        }

        var _this = (0, _possibleConstructorReturn3.default)(this, (WMSSource.__proto__ || (0, _getPrototypeOf2.default)(WMSSource)).call(this, source));

        _this.name = source.name;
        _this.zoom = source.zoom || { min: 0, max: 21 };
        _this.format = _this.format || 'image/png';
        _this.style = source.style || '';
        _this.width = source.heightMapWidth || 256;
        _this.version = source.version || '1.3.0';
        _this.transparent = source.transparent || false;

        if (!source.axisOrder) {
            // 4326 (lat/long) axis order depends on the WMS version used
            if (source.projection == 'EPSG:4326') {
                // EPSG 4326 x = lat, long = y
                // version 1.1.0 long/lat while version 1.3.0 mandates xy (so lat,long)
                _this.axisOrder = _this.version === '1.1.0' ? 'wsen' : 'swne';
            } else {
                // xy,xy order
                _this.axisOrder = 'wsen';
            }
        }

        var crsPropName = _this.version === '1.3.0' ? 'CRS' : 'SRS';

        _this.url = source.url + '?SERVICE=WMS&REQUEST=GetMap&LAYERS=' + _this.name + '&VERSION=' + _this.version + '&STYLES=' + _this.style + '&FORMAT=' + _this.format + '&TRANSPARENT=' + _this.transparent + '&BBOX=%bbox&' + crsPropName + '=' + _this.projection + '&WIDTH=' + _this.width + '&HEIGHT=' + _this.width;
        return _this;
    }

    (0, _createClass3.default)(WMSSource, [{
        key: 'urlFromExtent',
        value: function urlFromExtent(extent) {
            return _URLBuilder2.default.bbox(extent, this);
        }
    }, {
        key: 'extentInsideLimit',
        value: function extentInsideLimit(extent) {
            var localExtent = this.projection == extent.crs() ? extent : extent.as(this.projection);
            return (extent.zoom == undefined || !(extent.zoom < this.zoom.min || extent.zoom > this.zoom.max)) && this.extent.intersectsExtent(localExtent);
        }
    }]);
    return WMSSource;
}(_Source3.default);

exports.default = WMSSource;